@@ -396,6 +396,15 @@
           (recurse (%array-data x))
           (truly-the (integer 128 255) result))))))
 
+(declaim (ftype (sfunction (array) (values (integer 128 255) (unsigned-byte 8)))
+                array-underlying-widetag-and-shift))
+(defun array-underlying-widetag-and-shift (array)
+  (declare (explicit-check))
+  (let ((widetag (array-underlying-widetag array)))
+    (values widetag
+            (truly-the (unsigned-byte 8)
+                       (aref %%simple-array-n-bits-shifts%% widetag)))))
+
 ;; Complain in various ways about wrong MAKE-ARRAY and ADJUST-ARRAY arguments,
 ;; returning the two initialization arguments needed for DATA-VECTOR-FROM-INITS.
 ;; This is an unhygienic macro which would be a MACROLET other than for
@@ -452,10 +461,9 @@
 (defun %save-displaced-new-array-backpointer (array data)
   (flet ((purge (pointers)
            (remove-if-not #'weak-pointer-value pointers)))
-    (when (array-header-p data)
-      (setf (%array-displaced-from data)
-            (cons (make-weak-pointer array)
-                  (purge (%array-displaced-from data)))))))
+    (setf (%array-displaced-from data)
+          (cons (make-weak-pointer array)
+                (purge (%array-displaced-from data))))))
 
 (defmacro populate-dimensions (header list-or-index rank)
   `(if (listp ,list-or-index)
@@ -565,7 +573,8 @@
              (cond (displaced-to
                     (setf (%array-displacement array) (or displaced-index-offset 0))
                     (setf (%array-displaced-p array) t)
-                    (%save-displaced-new-array-backpointer array data))
+                    (when (adjustable-array-p data)
+                      (%save-displaced-new-array-backpointer array data)))
                    (t
                     (setf (%array-displaced-p array) nil)))
              (populate-dimensions array dimensions array-rank)
@@ -693,25 +702,30 @@ of specialized arrays is supported."
                     (make-array ,(1+ widetag-mask)))
                 (declaim (type (simple-array function (,(1+ widetag-mask)))
                                ,table-name))
-                (defmacro ,name (array-var)
-                  `(the function
-                     ;; Assigning TAG to 0 initially produces slightly better
-                     ;; code than would be generated by the more natural expression
-                     ;;   (let ((tag (if (%other-ptr ...) (widetag ...) 0)))
-                     ;; but either way is suboptimal. As expressed, if the array-var
-                     ;; is known to satisfy %other-pointer-p, then it performs a
-                     ;; move-immediate-to-register which is clobbered right away
-                     ;; by a zero-extending load. A peephole pass could eliminate
-                     ;; the first move as effectless.  If expressed the other way,
-                     ;; it would produce a jump around a jump because the compiler
-                     ;; is unwilling to *unconditionally* assign 0 into a register
-                     ;; to begin with. It actually wants to guard an immediate load
-                     ;; when it doesn't need to, as if both consequents of the IF
-                     ;; have side-effects that should not happen.
-                     (let ((tag 0))
-                       (when (%other-pointer-p ,array-var)
-                         (setf tag (%other-pointer-widetag ,array-var)))
-                       (svref ,',table-name tag)))))))
+                (defmacro ,name (array-var &optional vector-check)
+                  (if vector-check
+                      `(the function
+                            (svref ,',table-name (%other-pointer-widetag
+                                                  (locally (declare (optimize (safety 1)))
+                                                    (the vector ,array-var)))))
+                      `(the function
+                            ;; Assigning TAG to 0 initially produces slightly better
+                            ;; code than would be generated by the more natural expression
+                            ;;   (let ((tag (if (%other-ptr ...) (widetag ...) 0)))
+                            ;; but either way is suboptimal. As expressed, if the array-var
+                            ;; is known to satisfy %other-pointer-p, then it performs a
+                            ;; move-immediate-to-register which is clobbered right away
+                            ;; by a zero-extending load. A peephole pass could eliminate
+                            ;; the first move as effectless.  If expressed the other way,
+                            ;; it would produce a jump around a jump because the compiler
+                            ;; is unwilling to *unconditionally* assign 0 into a register
+                            ;; to begin with. It actually wants to guard an immediate load
+                            ;; when it doesn't need to, as if both consequents of the IF
+                            ;; have side-effects that should not happen.
+                            (let ((tag 0))
+                              (when (%other-pointer-p ,array-var)
+                                (setf tag (%other-pointer-widetag ,array-var)))
+                              (svref ,',table-name tag))))))))
   (def !find-data-vector-setter %%data-vector-setters%%)
   (def !find-data-vector-setter/check-bounds %%data-vector-setters/check-bounds%%)
   ;; Used by DO-VECTOR-DATA -- which in turn appears in DOSEQUENCE expansion,
@@ -739,8 +753,8 @@ of specialized arrays is supported."
                ,@decls
                (tagbody ,@forms))))))))
 
-(macrolet ((%ref (accessor-getter extra-params)
-             `(sb-c::%funcall-no-nargs (,accessor-getter array) array index ,@extra-params))
+(macrolet ((%ref (accessor-getter extra-params &optional vector-check)
+             `(sb-c::%funcall-no-nargs (,accessor-getter array ,vector-check) array index ,@extra-params))
            (define (accessor-name slow-accessor-name accessor-getter
                                   extra-params check-bounds)
              `(progn
@@ -755,6 +769,10 @@ of specialized arrays is supported."
                                      ;; dispatch.
                                      (safety 0)))
                   (%ref ,accessor-getter ,extra-params))
+                (defun ,(symbolicate 'vector- accessor-name) (array index ,@extra-params)
+                  (declare (explicit-check)
+                           (optimize speed (safety 0)))
+                  (%ref ,accessor-getter ,extra-params t))
                 (defun ,slow-accessor-name (array index ,@extra-params)
                   (declare (optimize speed (safety 0))
                            (array array))
@@ -806,9 +824,9 @@ of specialized arrays is supported."
                            (ignorable index))
                   ,(if type
                        `(data-vector-ref (the ,atype vector)
-                                         (locally
-                                             (declare (optimize (safety 1)))
-                                           (the index
+                                         (the index
+                                              (locally
+                                                  (declare (optimize (safety 1)))
                                                 (,@check-form index))))
                        `(data-nil-vector-ref (the ,atype vector) index)))))
            (define-setter (saetp check-form)
@@ -830,7 +848,7 @@ of specialized arrays is supported."
                                        ;; a cheaper one.
                                        (declare (optimize (speed 1)
                                                           (safety 1)))
-                                     (the* (,type :context :aref) new-value)))
+                                     (the* (,type :context sb-c::aref-context) new-value)))
                   ;; Low-level setters return no value
                   new-value)))
            (define-reffers (symbol deffer check-form slow-path)
@@ -852,7 +870,6 @@ of specialized arrays is supported."
                         collect `(setf (svref ,symbol ,widetag)
                                        (,deffer ,saetp ,check-form))))))
   (defun !hairy-data-vector-reffer-init ()
-    (!blt-copiers-cold-init)
     (define-reffers %%data-vector-reffers%% define-reffer
       (progn)
       #'slow-hairy-data-vector-ref)
@@ -882,8 +899,7 @@ of specialized arrays is supported."
        (consp (%array-displaced-p array))))
 
 (declaim (ftype (function (array) nil) invalid-array-error))
-(defun invalid-array-error (array)
-  (declare (optimize allow-non-returning-tail-call))
+(define-error-wrapper invalid-array-error (array)
   (aver (array-header-p array))
   ;; Array invalidation stashes the original dimensions here...
   (let ((dims (%array-displaced-p array))
@@ -897,8 +913,7 @@ of specialized arrays is supported."
 
 (declaim (ftype (function (array t integer &optional t) nil)
                 invalid-array-index-error))
-(defun invalid-array-index-error (array index bound &optional axis)
-  (declare (optimize allow-non-returning-tail-call))
+(define-error-wrapper invalid-array-index-error (array index bound &optional axis)
   (if (invalid-array-p array)
       (invalid-array-error array)
       (error 'invalid-array-index-error
@@ -909,7 +924,7 @@ of specialized arrays is supported."
 
 ;;; SUBSCRIPTS has a dynamic-extent list structure and is destroyed
 (defun %array-row-major-index (array &rest subscripts)
-  (declare (truly-dynamic-extent subscripts)
+  (declare (dynamic-extent subscripts)
            (array array))
   (let ((length (length subscripts)))
     (cond ((array-header-p array)
@@ -943,7 +958,7 @@ of specialized arrays is supported."
 
 (defun array-in-bounds-p (array &rest subscripts)
   "Return T if the SUBSCRIPTS are in bounds for the ARRAY, NIL otherwise."
-  (declare (truly-dynamic-extent subscripts))
+  (declare (dynamic-extent subscripts))
   (let ((length (length subscripts)))
     (cond ((array-header-p array)
            (let ((rank (%array-rank array)))
@@ -964,12 +979,12 @@ of specialized arrays is supported."
                      (length (truly-the (simple-array * (*)) array)))))))))
 
 (defun array-row-major-index (array &rest subscripts)
-  (declare (truly-dynamic-extent subscripts))
+  (declare (dynamic-extent subscripts))
   (apply #'%array-row-major-index array subscripts))
 
 (defun aref (array &rest subscripts)
   "Return the element of the ARRAY specified by the SUBSCRIPTS."
-  (declare (truly-dynamic-extent subscripts))
+  (declare (dynamic-extent subscripts))
   (row-major-aref array (apply #'%array-row-major-index array subscripts)))
 
 ;;; (setf aref/bit/sbit) are implemented using setf-functions,
@@ -978,7 +993,7 @@ of specialized arrays is supported."
 ;;; haven't found technical advantages or disadvantages for either
 ;;; scheme.
 (defun (setf aref) (new-value array &rest subscripts)
-  (declare (truly-dynamic-extent subscripts)
+  (declare (dynamic-extent subscripts)
            (type array array))
   (setf (row-major-aref array (apply #'%array-row-major-index array subscripts))
         new-value))
@@ -1005,14 +1020,14 @@ of specialized arrays is supported."
 (defun bit (bit-array &rest subscripts)
   "Return the bit from the BIT-ARRAY at the specified SUBSCRIPTS."
   (declare (type (array bit) bit-array)
-           (truly-dynamic-extent subscripts)
+           (dynamic-extent subscripts)
            (optimize (safety 1)))
   (row-major-aref bit-array (apply #'%array-row-major-index bit-array subscripts)))
 
 (defun (setf bit) (new-value bit-array &rest subscripts)
   (declare (type (array bit) bit-array)
            (type bit new-value)
-           (truly-dynamic-extent subscripts)
+           (dynamic-extent subscripts)
            (optimize (safety 1)))
   (setf (row-major-aref bit-array
                         (apply #'%array-row-major-index bit-array subscripts))
@@ -1021,7 +1036,7 @@ of specialized arrays is supported."
 (defun sbit (simple-bit-array &rest subscripts)
   "Return the bit from SIMPLE-BIT-ARRAY at the specified SUBSCRIPTS."
   (declare (type (simple-array bit) simple-bit-array)
-           (truly-dynamic-extent subscripts)
+           (dynamic-extent subscripts)
            (optimize (safety 1)))
   (row-major-aref simple-bit-array
                   (apply #'%array-row-major-index simple-bit-array subscripts)))
@@ -1029,7 +1044,7 @@ of specialized arrays is supported."
 (defun (setf sbit) (new-value bit-array &rest subscripts)
   (declare (type (simple-array bit) bit-array)
            (type bit new-value)
-           (truly-dynamic-extent subscripts)
+           (dynamic-extent subscripts)
            (optimize (safety 1)))
   (setf (row-major-aref bit-array
                         (apply #'%array-row-major-index bit-array subscripts))
@@ -1046,6 +1061,7 @@ of specialized arrays is supported."
 
 (defun array-element-type (array)
   "Return the type of the elements of the array"
+  (declare (explicit-check array))
   (truly-the (or list symbol)
              (widetag->element-type (array-underlying-widetag array))))
 
@@ -1115,35 +1131,34 @@ of specialized arrays is supported."
   "Return T if the given ARRAY has a fill pointer, or NIL otherwise."
   (array-has-fill-pointer-p array))
 
-(defun fill-pointer-error (vector)
-  (declare (optimize allow-non-returning-tail-call)
-           (optimize (sb-c::verify-arg-count 0)))
+(define-error-wrapper fill-pointer-error (vector)
+  (declare (optimize (sb-c::verify-arg-count 0)))
   (error 'simple-type-error
          :datum vector
          :expected-type '(and vector (satisfies array-has-fill-pointer-p))
          :format-control "~S is not an array with a fill pointer."
          :format-arguments (list vector)))
 
-(declaim (inline fill-pointer))
+
 (defun fill-pointer (vector)
   "Return the FILL-POINTER of the given VECTOR."
   (declare (explicit-check))
-  (if (array-has-fill-pointer-p vector)
-      (%array-fill-pointer vector)
-      (fill-pointer-error vector)))
+  (fill-pointer vector))
 
 (defun %set-fill-pointer (vector new)
   (declare (explicit-check))
-  (cond ((not (array-has-fill-pointer-p vector)) (fill-pointer-error vector))
-        ((> (the index new) (%array-available-elements vector))
-         (let ((max (%array-available-elements vector)))
-           (error 'simple-type-error
-                  :datum new
-                  :expected-type (list 'integer 0 max)
-                  :format-control "The new fill pointer, ~S, is larger than the length of the vector (~S.)"
-                  :format-arguments (list new max))))
+  (cond ((not (and (arrayp vector)
+                   (array-has-fill-pointer-p vector)))
+         (fill-pointer-error vector))
         (t
-         (setf (%array-fill-pointer vector) new))))
+         (let ((max (%array-available-elements vector)))
+           (when (> (the (and unsigned-byte fixnum) new) max)
+             (error 'simple-type-error
+                    :datum new
+                    :expected-type (list 'integer 0 max)
+                    :format-control "The new fill pointer, ~S, is larger than the length of the vector (~S.)"
+                    :format-arguments (list new max)))
+           (setf (%array-fill-pointer vector) (truly-the index new))))))
 
 ;;; FIXME: It'd probably make sense to use a MACROLET to share the
 ;;; guts of VECTOR-PUSH between VECTOR-PUSH-EXTEND. Such a macro
@@ -1165,21 +1180,34 @@ of specialized arrays is supported."
            (setf (%array-fill-pointer array) (1+ fill-pointer))
            fill-pointer))))
 
-(defun !blt-copiers-cold-init ()
-  (let ((array (make-array 32 :initial-element nil)))
-    (macrolet ((init ()
-                 `(progn
-                    ,@(loop for saetp across *specialized-array-element-type-properties*
-                            when (and (not (member (saetp-specifier saetp) '(t nil)))
-                                      (<= (saetp-n-bits saetp) n-word-bits))
-                              collect `(setf (svref array ,(ash (- (saetp-typecode saetp) 128) -2))
-                                             #',(intern (format nil "UB~D-BASH-COPY"
-                                                                (saetp-n-bits saetp))
-                                                        "SB-KERNEL"))))))
-      (init))
-    (setf (fdefinition 'blt-copier-for-widetag)
-          (lambda (x)
-            (aref array (ash (- x 128) -2))))))
+#-system-tlabs
+(defmacro reallocate-vector-with-widetag (old-vector &rest args)
+  (declare (ignore old-vector))
+  `(allocate-vector-with-widetag ,@args))
+
+;;; This does not try to allow for resizing (ARRAY NIL) - there's no backing storage anyway.
+;;; However, ADJUST-ARRAY apparently thinks it can resize non-simple arrays of
+;;; element type NIL, but fails in ZAP-ARRAY-DATA-AUX. e.g.:
+;;;   (adjust-array (make-array '(10 10) :element-type nil) '(20 20))
+;;; allocates a non-simple 10x10 array pointing to a (SIMPLE-ARRAY NIL 100)
+;;; and then gets "An attempt to access an array of element-type NIL was made"
+;;; because it doesn't know not to try to copy elements.
+;;; So unless we think that that is one of the most pressing issues that demands
+;;; a fix, who cares how we reallocate?
+;;; If you're manipulating such arrays, quite literally you deserve to lose.
+;;; FIXME: does not support #+ubsan, which is fairly bit-rotted, so ... meh.
+#+system-tlabs
+(defun reallocate-vector-with-widetag (old-vector widetag length n-bits-shift)
+  (declare (type (unsigned-byte 8) widetag)
+           (type index length))
+  ;; KLUDGE: add SAETP-N-PAD-ELEMENTS "by hand" since there is
+  ;; but a single case involving it now.
+  (let* ((full-length (+ length (if (= widetag simple-base-string-widetag) 1 0)))
+         (nwords (the fixnum (vector-length-in-words full-length n-bits-shift))))
+    (if (sb-vm::force-to-heap-p old-vector)
+        (locally (declare (sb-c::tlab :system))
+          (allocate-vector widetag length nwords))
+        (allocate-vector widetag length nwords))))
 
 (defun extend-vector (vector min-extension)
   (declare (optimize speed)
@@ -1198,8 +1226,8 @@ of specialized arrays is supported."
              (n-bits-shift (aref %%simple-array-n-bits-shifts%% widetag))
              (new-data
               ;; FIXME: mark prefix of shadow bits assigned, suffix unassigned
-              (allocate-vector-with-widetag #+ubsan nil
-                                            widetag new-length n-bits-shift)))
+              (reallocate-vector-with-widetag old-data #+ubsan nil
+                                              widetag new-length n-bits-shift)))
         ;; Copy the data
         (if (= widetag simple-vector-widetag) ; the most common case
             (replace (truly-the simple-vector new-data) ; transformed
@@ -1329,9 +1357,10 @@ of specialized arrays is supported."
                       ;; if total size is unchanged, and it was not a displaced array,
                       ;; then this array owns the data and can retain it.
                       old-data
-                      (let ((data (allocate-vector-with-widetag #+ubsan t
-                                                                widetag new-total-size
-                                                                n-bits-shift)))
+                      (let ((data
+                             (reallocate-vector-with-widetag old-data #+ubsan t
+                                                             widetag new-total-size
+                                                             n-bits-shift)))
                         (replace data old-data
                                  :start1 0 :end1 new-total-size
                                  :start2 old-start :end2 old-end)
@@ -1796,7 +1825,7 @@ function to be removed without further warning."
 ;;; Finally, the DISPATCH-FOO macro is defined which does the actual
 ;;; dispatching when called. It expects arguments that match PARAMS.
 ;;;
-(defmacro sb-impl::!define-array-dispatch (style dispatch-name params &body body)
+(defmacro sb-impl::!define-array-dispatch (style dispatch-name params nil-array &body body)
   #-(or x86 x86-64) (setq style :call)
   (let ((table-name (symbolicate "%%" dispatch-name "-FUNS%%"))
         (error-name (symbolicate "HAIRY-" dispatch-name "-ERROR")))
@@ -1833,10 +1862,12 @@ function to be removed without further warning."
                             (defun ,fun-name ,params
                               (declare (type (simple-array ,specifier (*))
                                              ,(first params)))
-                              ,@body)
+                              ,@(if (null specifier)
+                                    nil-array
+                                    body))
                             (setf (svref ,table-name ,typecode) #',fun-name))))
        (defmacro ,dispatch-name (&rest args)
-         (check-type (first args) symbol)
+         (aver (symbolp (first args)))
          (let ((tag (gensym "TAG")))
            `(funcall
              (truly-the function
@@ -1887,32 +1918,3 @@ function to be removed without further warning."
 ;;; Horrible kludge for the "static-vectors" system
 ;;; which uses an internal symbol in SB-IMPL.
 (import '%vector-widetag-and-n-bits-shift 'sb-impl)
-
-(defun make-weak-vector (length &key (initial-contents nil contents-p)
-                                     (initial-element nil element-p))
-  (declare (index length))
-  (when (and element-p contents-p)
-    (error "Can't specify both :INITIAL-ELEMENT and :INITIAL-CONTENTS"))
-  ;; Explicitly compute a widetag with the weakness bit ORed in.
-  (let ((type (logior (ash vector-weak-flag array-flags-position) simple-vector-widetag)))
-    ;; These allocation calls are the transforms of MAKE-ARRAY for a vector with
-    ;; the respective initializing keyword arg. This is badly OAOO-violating and
-    ;; almost makes me want to cry, but not quite enough for me to improve it.
-    (if contents-p
-        (let ((contents-length (length initial-contents)))
-          (if (= length contents-length)
-              (replace (truly-the simple-vector
-                                  (allocate-vector #+ubsan nil type length length))
-                       initial-contents)
-              (error "~S has ~D elements, vector length is ~D."
-                     :initial-contents contents-length length)))
-        (fill (truly-the simple-vector
-                         (allocate-vector #+ubsan nil type length length))
-              ;; 0 is the usual default, but NIL makes more sense for weak vectors
-              ;; as it is the value assigned to broken hearts.
-              (if element-p initial-element nil)))))
-
-(defun weak-vector-p (x)
-  (and (simple-vector-p x)
-       #+(or x86 x86-64) (test-header-data-bit x (ash vector-weak-flag array-flags-data-position))
-       #-(or x86 x86-64) (logtest (get-header-data x) (ash vector-weak-flag array-flags-data-position))))
