@@ -15,12 +15,12 @@
 
 (declaim (inline dynamic-usage))
 (defun dynamic-usage ()
+  #+mark-region-gc
+  (let ((bytes (extern-alien "bytes_allocated" os-vm-size-t)))
+    (values bytes
+            (- (* (pages-allocated) sb-vm:gencgc-page-bytes) bytes)))
   #+gencgc
-  (extern-alien "bytes_allocated" os-vm-size-t)
-  #-gencgc
-  (truly-the word
-             (- (sap-int (sb-c::dynamic-space-free-pointer))
-                sb-vm:dynamic-space-start)))
+  (extern-alien "bytes_allocated" os-vm-size-t))
 
 (defun static-space-usage ()
   (- (sap-int sb-vm:*static-space-free-pointer*) sb-vm:static-space-start))
@@ -56,7 +56,8 @@
   ;; and we need a function in C to do that.
   (gc)
   (setf *n-bytes-freed-or-purified* 0
-        *gc-run-time* 0))
+        *gc-run-time* 0
+        *gc-real-time* 0))
 
 (declaim (ftype (sfunction () unsigned-byte) get-bytes-consed))
 (defun get-bytes-consed ()
@@ -125,21 +126,32 @@ run in any thread.")
      (alien-funcall (extern-alien "release_gc_lock" (function void)))
      t))
 
+(defmacro get-gc-real-time ()
+  ;; 64-bit is just a call to clock-gettime and some math that won't ever cons
+  #+64-bit '(get-internal-real-time)
+  ;; 32-bit memoizes the result to try to avoid bignum consing,
+  ;; which can fail during foreign thread creation.
+  #-64-bit 0)
+
 (defun sub-gc (gen)
   (cond (*gc-inhibit*
          (setf *gc-pending* t)
          nil)
         (t
          (flet ((perform-gc ()
+                  (declare (sb-c::tlab :system)) ; for *gc-epoch*
                   ;; Called from WITHOUT-GCING and WITHOUT-INTERRUPTS
                   ;; after the world has been stopped, but it's an
                   ;; awkwardly long piece of code to nest so deeply.
                   (let ((old-usage (dynamic-usage))
                         (new-usage 0)
-                        (start-time (get-internal-run-time)))
+                        (start-time (get-internal-run-time))
+                        (start-real-time (get-gc-real-time)))
                     (collect-garbage gen)
                     (setf *gc-epoch* (cons 0 0))
-                    (let ((run-time (- (get-internal-run-time) start-time)))
+                    (let ((run-time (- (get-internal-run-time) start-time))
+                          (real-time (- (get-gc-real-time) start-real-time)))
+                      (incf *gc-real-time* real-time)
                       ;; KLUDGE: Sometimes we see the second getrusage() call
                       ;; return a smaller value than the first, which can
                       ;; lead to *GC-RUN-TIME* to going negative, which in
@@ -221,7 +233,8 @@ run in any thread.")
   ;; which is an arbitrary one. If those actions aquire any locks, or are sensitive
   ;; to the state of *ALLOW-WITH-INTERRUPTS*, any deadlocks of what-have-you
   ;; are user error. Hooks need to be sufficiently uncomplicated as to be harmless.
-  (call-hooks "after-GC" *after-gc-hooks* :on-error :warn))
+  (sb-vm:without-arena "post-gc"
+    (call-hooks "after-GC" *after-gc-hooks* :on-error :warn)))
 
 #-sb-thread
 (defun post-gc ()
@@ -279,36 +292,20 @@ run in any thread.")
 
 ;;; This is the user-advertised garbage collection function.
 (defun gc (&key (full nil) (gen 0) &allow-other-keys)
-  #+gencgc
   "Initiate a garbage collection.
 
 The default is to initiate a nursery collection, which may in turn
 trigger a collection of one or more older generations as well. If FULL
 is true, all generations are collected. If GEN is provided, it can be
-used to specify the oldest generation guaranteed to be collected.
-
-On CheneyGC platforms arguments FULL and GEN take no effect: a full
-collection is always performed."
-  #-gencgc
-  "Initiate a garbage collection.
-
-The collection is always a full collection.
-
-Arguments FULL and GEN can be used for compatibility with GENCGC
-platforms: there the default is to initiate a nursery collection,
-which may in turn trigger a collection of one or more older
-generations as well. If FULL is true, all generations are collected.
-If GEN is provided, it can be used to specify the oldest generation
-guaranteed to be collected."
-  #-gencgc (declare (ignore full))
-  (let (#+gencgc (gen (if full sb-vm:+pseudo-static-generation+ gen)))
+used to specify the oldest generation guaranteed to be collected."
+  (let ((gen (if full sb-vm:+pseudo-static-generation+ gen)))
     (when (eq t (sub-gc gen))
       (post-gc))))
 
 (define-alien-routine scrub-control-stack void)
 
 (defun unsafe-clear-roots (gen)
-  #-gencgc (declare (ignore gen))
+  (declare (ignorable gen))
   ;; KLUDGE: Do things in an attempt to get rid of extra roots. Unsafe
   ;; as having these cons more than we have space left leads to huge
   ;; badness.
@@ -317,7 +314,6 @@ guaranteed to be collected."
   ;; removes duplicate entries.
   (scrub-power-cache)
   ;; Clear caches depending on the generation being collected.
-  #+gencgc
   (cond ((eql 0 gen)
          ;; Drop strings because the hash is pointer-hash
          ;; but there is no automatic cache rehashing after GC.
@@ -325,9 +321,7 @@ guaranteed to be collected."
         ((eql 1 gen)
          (sb-format::tokenize-control-string-cache-clear))
         (t
-         (drop-all-hash-caches)))
-  #-gencgc
-  (drop-all-hash-caches))
+         (drop-all-hash-caches))))
 
 ;;;; auxiliary functions
 
@@ -396,7 +390,7 @@ statistics are appended to it."
     (array generation #.(1+ sb-vm:+pseudo-static-generation+)))
 
 ;;; Why is PAGE-INDEX-T in SB-KERNEL but PAGE and the page table are in SB-VM?
-(define-alien-type (struct sb-vm::page)
+(define-alien-type nil
     (struct sb-vm::page
             ;; To cut down the size of the page table, the scan_start_offset
             ;; - a/k/a "start" - is measured in 4-byte integers regardless
@@ -412,6 +406,11 @@ statistics are appended to it."
 (declaim (inline sb-vm:find-page-index))
 (define-alien-routine ("ext_find_page_index" sb-vm:find-page-index) page-index-t (address unsigned))
 
+(defun pages-allocated ()
+  (loop for n below (extern-alien "next_free_page" signed)
+        count (not (zerop (slot (deref sb-vm:page-table n) 'sb-vm::flags)))))
+
+#-mark-region-gc
 (defun generation-of (object)
   (with-pinned-objects (object)
     (let* ((addr (get-lisp-obj-address object))
@@ -426,6 +425,14 @@ statistics are appended to it."
                (logand (if (fdefn-p object) (sap-ref-8 sap 1) (sap-ref-8 sap 3))
                        #xF)))))))
 
+#+mark-region-gc
+(defun generation-of (object)
+  (with-alien ((gc-gen-of (function char unsigned int) :extern))
+    (let ((result (with-pinned-objects (object)
+                    (alien-funcall gc-gen-of
+                                   (get-lisp-obj-address object) 127))))
+      (unless (= result 127) result))))
+
 (export 'page-protected-p)
 (macrolet ((addr->mark (addr)
              `(sap-ref-8 (extern-alien "gc_card_mark" system-area-pointer)
@@ -518,6 +525,12 @@ Experimental: interface subject to change."
                     ((< sb-vm:read-only-space-start addr
                         (sap-int sb-vm:*read-only-space-free-pointer*))
                      :read-only)
+                    ;; Without immobile-space, the text range is
+                    ;; more-or-less an extension of static space.
+                    #-immobile-space
+                    ((< sb-vm:text-space-start addr
+                        (sap-int sb-vm:*text-space-free-pointer*))
+                     :static)
                     ((< sb-vm:static-space-start addr
                         (sap-int sb-vm:*static-space-free-pointer*))
                      :static))))
