@@ -20,6 +20,8 @@
                         sb-vm:*read-only-space-free-pointer*)
     system-area-pointer)
 
+#+relocatable-static-space
+(define-alien-variable ("STATIC_SPACE_START" sb-vm:static-space-start) sb-kernel::os-vm-size-t)
 ;;; STATIC-SPACE-START is a constant from genesis
 (define-alien-variable ("static_space_free_pointer" sb-vm:*static-space-free-pointer*)
   system-area-pointer)
@@ -49,11 +51,11 @@
     (let ((addr (get-lisp-obj-address x)))
       (< sb-vm:dynamic-space-start addr (sap-int (dynamic-space-free-pointer))))))
 
+(define-alien-variable ("TEXT_SPACE_START" sb-vm:text-space-start) unsigned-long)
+
 #+immobile-space
-(progn
 (define-symbol-macro sb-vm:alien-linkage-table-space-start
     (extern-alien "ALIEN_LINKAGE_TABLE_SPACE_START" unsigned))
-(define-alien-variable ("TEXT_SPACE_START" sb-vm:text-space-start) unsigned-long))
 
 #+darwin-jit
 (define-alien-variable ("static_code_space_free_pointer" sb-vm:*static-code-space-free-pointer*)
@@ -83,6 +85,10 @@
 (setf (documentation 'get-errno 'function)
       "Return the value of the C library pseudo-variable named \"errno\".")
 
+(define-alien-routine ("os_set_errno" set-errno) void (new-errno int))
+(setf (documentation 'set-errno 'function)
+      "Set the C library pseudo-variable named \"errno\", for obscure syscalls.")
+
 ;;; Decode errno into a string.
 #-win32
 (defun strerror (&optional (errno (get-errno)))
