@@ -36,31 +36,35 @@
                                       (octets-encoding-error-position c)))
                      (octets-encoding-error-external-format c)))))
 
-(declaim (ftype (sfunction (t t t) (simple-array (unsigned-byte 8) 1))
+(defun encoding-replacement-octetify (thing external-format)
+  (etypecase thing
+    ;; TODO: make sure EXTERNAL-FORMAT here does not have a REPLACEMENT, and handle
+    ;; encoding errors explicitly
+    (character
+     (string-to-octets (string thing) :external-format external-format))
+    (string
+     (string-to-octets thing :external-format external-format))
+    ((unsigned-byte 8) (make-array 1 :element-type '(unsigned-byte 8) :initial-element thing))
+    (sequence (coerce thing '(simple-array (unsigned-byte 8) 1)))))
+
+(declaim (ftype (sfunction (t t t t) (simple-array (unsigned-byte 8) 1))
                 encoding-error))
-(defun encoding-error (external-format string pos)
-  (restart-case
-      (error 'octets-encoding-error
-             :external-format external-format
-             :string string
-             :position pos)
-    (use-value (replacement)
-      :report "Supply a set of bytes to use in place of the invalid one."
-      :interactive
-      (lambda ()
-        (read-evaluated-form
-         "Replacement byte, bytes, character, or string (evaluated): "))
-      (typecase replacement
-        ((unsigned-byte 8)
-         (make-array 1 :element-type '(unsigned-byte 8) :initial-element replacement))
-        (character
-         (string-to-octets (string replacement)
-                           :external-format external-format))
-        (string
-         (string-to-octets replacement
-                           :external-format external-format))
-        (t
-         (coerce replacement '(simple-array (unsigned-byte 8) (*))))))))
+(defun encoding-error (external-format replacement string pos)
+  (flet ((replacement (replacement) (encoding-replacement-octetify replacement external-format)))
+    (if replacement
+        (replacement replacement)
+        (restart-case
+            (error 'octets-encoding-error
+                   :external-format external-format
+                   :string string
+                   :position pos)
+          (use-value (replacement)
+            :report "Supply a set of bytes to use in place of the invalid one."
+            :interactive
+            (lambda ()
+              (read-evaluated-form
+               "Replacement byte, bytes, character, or string (evaluated): "))
+            (replacement replacement))))))
 
 ;;; decoding condition
 
@@ -98,23 +102,37 @@
 (define-condition invalid-utf8-continuation-byte (octet-decoding-error) ())
 (define-condition overlong-utf8-sequence (octet-decoding-error) ())
 
-(define-condition malformed-ascii (octet-decoding-error) ())
+(defun decoding-replacement-stringify (thing external-format)
+  (etypecase thing
+    (character (string thing))
+    (string thing)
+    ;; TODO: make sure EXTERNAL-FORMAT here does not have a REPLACEMENT, and handle
+    ;; decoding errors explicitly
+    ((unsigned-byte 8)
+     (let ((octets (make-array 1 :element-type '(unsigned-byte 8) :initial-element thing)))
+       (octets-to-string octets :external-format external-format)))
+    (sequence
+     (let ((octets (coerce thing '(simple-array (unsigned-byte 8) 1))))
+       (octets-to-string octets :external-format external-format)))))
 
-(defun decoding-error (array start end external-format reason pos)
-  (restart-case
-      (error reason
-             :external-format external-format
-             :array array
-             :start start
-             :end end
-             :pos pos)
-    (use-value (s)
-      :report "Supply a replacement string designator."
-      :interactive
-      (lambda ()
-        (read-evaluated-form
-         "Enter a replacement string designator (evaluated): "))
-      (string s))))
+(defun decoding-error (array start end external-format replacement reason pos)
+  (flet ((replacement (thing) (decoding-replacement-stringify thing external-format)))
+    (if replacement
+        (replacement replacement)
+        (restart-case
+            (error reason
+                   :external-format external-format
+                   :array array
+                   :start start
+                   :end end
+                   :pos pos)
+          (use-value (replacement)
+            :report "Supply a replacement string."
+            :interactive
+            (lambda ()
+              (read-evaluated-form
+               "Replacement byte, bytes, character, or string (evaluated): "))
+            (replacement replacement))))))
 
 ;;; Utilities used in both to-string and to-octet conversions
 
@@ -224,17 +242,17 @@
                                       nil))))))))
 
 (declaim (inline get-latin-bytes))
-(defun get-latin-bytes (mapper external-format string pos)
+(defun get-latin-bytes (mapper external-format replacement string pos)
   (let ((code (funcall mapper (char-code (char string pos)))))
     (declare (type (or null char-code) code))
     (values (cond
               ((and code (< code 256)) code)
               (t
-               (encoding-error external-format string pos)))
+               (encoding-error external-format replacement string pos)))
             1)))
 
 (declaim (inline string->latin%))
-(defun string->latin% (string sstart send get-bytes null-padding)
+(defun string->latin% (string sstart send get-bytes null-padding replacement)
   (declare (optimize speed)
            (type simple-string string)
            (type index sstart send)
@@ -253,7 +271,7 @@
     (tagbody
      :no-error
        (loop for pos of-type index from sstart below send
-          do (let ((byte (funcall get-bytes string pos)))
+          do (let ((byte (funcall get-bytes string pos replacement)))
                (typecase byte
                  ((unsigned-byte 8)
                   (locally (declare (optimize (sb-c:insert-array-bounds-checks 0)))
@@ -282,7 +300,7 @@
                       (vector-push-extend (aref thing i) new-octets))))))
            (extend error-replacement)
            (loop for pos of-type index from (1+ error-position) below send
-                 do (extend (funcall get-bytes string pos))
+                 do (extend (funcall get-bytes string pos replacement))
                  finally (return-from string->latin%
                            (progn
                              (unless (zerop null-padding)
@@ -330,6 +348,7 @@
 (defvar *default-external-format* :utf-8)
 
 (defun default-external-format ()
+  (/show0 "/getting default external format")
   *default-external-format*)
 
 
@@ -340,12 +359,20 @@
                                    (default-external-format)
                                    external-format)))
 
-(declaim (ftype (sfunction ((vector (unsigned-byte 8)) &key (:external-format t)
-                                   (:start index)
-                                   (:end sequence-end))
+(declaim (inline %octets-to-string))
+(declaim (ftype (sfunction (function (vector (unsigned-byte 8)) index sequence-end t)
                            (or (simple-array character (*))
                                (simple-array base-char (*))))
-                octets-to-string))
+                %octets-to-string))
+(defun %octets-to-string (fun vector start end replacement)
+  (declare (explicit-check start end :result))
+  (with-array-data ((vector vector)
+                    (start start)
+                    (end end)
+                    :check-fill-pointer t)
+    (declare (type (simple-array (unsigned-byte 8) (*)) vector))
+    (funcall fun vector start end replacement)))
+(declaim (notinline %octets-to-string))
 (defun octets-to-string (vector &key (external-format :default) (start 0) end)
   "Return a string obtained by decoding VECTOR according to EXTERNAL-FORMAT.
 
@@ -361,21 +388,24 @@ SB-INT:CHARACTER-DECODING-ERROR is signaled.
 Note that for some values of EXTERNAL-FORMAT the length of the
 returned string may be different from the length of VECTOR (or the
 subsequence bounded by START and END)."
+  (let* ((ef (maybe-defaulted-external-format external-format))
+         (replacement (ef-replacement ef)))
+    (declare (inline %octets-to-string))
+    (%octets-to-string (ef-octets-to-string-fun ef) vector start end replacement)))
+
+(declaim (inline %string-to-octets))
+(declaim (ftype (sfunction (function string index sequence-end t t)
+                           (simple-array (unsigned-byte 8) (*)))
+                %string-to-octets))
+(defun %string-to-octets (fun string start end null-terminate replacement)
   (declare (explicit-check start end :result))
-  (with-array-data ((vector vector)
+  (with-array-data ((string string)
                     (start start)
                     (end end)
                     :check-fill-pointer t)
-    (declare (type (simple-array (unsigned-byte 8) (*)) vector))
-    (let ((ef (maybe-defaulted-external-format external-format)))
-      (funcall (ef-octets-to-string-fun ef) vector start end))))
-
-(declaim (ftype (sfunction (string &key (:external-format t)
-                                   (:start index)
-                                   (:end sequence-end)
-                                   (:null-terminate t))
-                           (simple-array (unsigned-byte 8) (*)))
-                string-to-octets))
+    (declare (type simple-string string))
+    (funcall fun string start end null-terminate replacement)))
+(declaim (notinline %string-to-octets))
 (defun string-to-octets (string &key (external-format :default)
                          (start 0) end null-terminate)
   "Return an octet vector that is STRING encoded according to EXTERNAL-FORMAT.
@@ -396,14 +426,11 @@ Note that for some values of EXTERNAL-FORMAT and NULL-TERMINATE the
 length of the returned vector may be different from the length of
 STRING (or the subsequence bounded by START and END)."
   (declare (explicit-check start end :result))
-  (with-array-data ((string string)
-                    (start start)
-                    (end end)
-                    :check-fill-pointer t)
-    (declare (type simple-string string))
-    (let ((ef (maybe-defaulted-external-format external-format)))
-      (funcall (ef-string-to-octets-fun ef) string start end
-               (if null-terminate 1 0)))))
+  (let* ((ef (maybe-defaulted-external-format external-format))
+         (replacement (ef-replacement ef)))
+    (declare (inline %string-to-octets))
+    (%string-to-octets (ef-string-to-octets-fun ef) string start end
+                       (if null-terminate 1 0) replacement)))
 
 ;;; Vector of all available EXTERNAL-FORMAT instances. Each format is named
 ;;; by one or more keyword symbols. The mapping from symbol to index into this
@@ -414,77 +441,53 @@ STRING (or the subsequence bounded by START and END)."
   ;; TODO: compare-and-swap the entry if NAME already has an index
   ;; specifying to demand-load this format from a fasl.
   ;; All synonyms of that name will also references the loaded format.
-  (let* ((entry (apply #'%make-external-format :names names args))
-         (table *external-formats*)
-         (free-index (or (get (car names) :external-format)
-                         (position nil table))))
+  (let* ((table *external-formats*)
+         (newline-variant (getf args :newline-variant))
+         (index (get (car names) :external-format))
+         (current-entry (and index (aref table index)))
+         (canonical-ef (if (consp current-entry) (car current-entry) current-entry))
+         (names (if canonical-ef (ef-names canonical-ef) names))
+         (ef (apply #'%make-external-format :names names args))
+         (free-index (or index (position nil table))))
+    (unless (eql newline-variant :lf)
+      (aver index)
+      (aver current-entry)
+      (unless (consp current-entry)
+        (setf current-entry (list current-entry)))
+      (setf (aref table index)
+            (cons (car current-entry)
+                  (acons (list newline-variant) ef (cdr current-entry))))
+      (return-from register-external-format))
     (dolist (name names)
       (setf (get name :external-format) free-index))
-    (setf (aref table free-index) entry)))
+    (setf (aref table free-index) ef)))
 
-(defun wrap-external-format-functions (external-format fun)
-  (let ((result (%copy-external-format external-format)))
-    (macrolet ((frob (accessor)
-                 `(setf (,accessor result) (funcall fun (,accessor result)))))
-      (frob ef-read-n-chars-fun)
-      (frob ef-read-char-fun)
-      (frob ef-write-n-bytes-fun)
-      (frob ef-write-char-none-buffered-fun)
-      (frob ef-write-char-line-buffered-fun)
-      (frob ef-write-char-full-buffered-fun)
-      (frob ef-resync-fun)
-      (frob ef-bytes-for-char-fun)
-      (frob ef-read-c-string-fun)
-      (frob ef-write-c-string-fun)
-      (frob ef-octets-to-string-fun)
-      (frob ef-string-to-octets-fun))
-    result))
 ;;; This function was moved from 'fd-stream' because it depends on
 ;;; the various error classes, two of which are defined just above.
 ;;; XXX: Why does this get called with :DEFAULT and NIL when neither is
-;;; the name  of any format? Shouldn't those be handled higher up,
+;;; the name of any format? Shouldn't those be handled higher up,
 ;;; or else this should return the actual default?
 (defun get-external-format (external-format)
-  (flet ((replacement-handlerify (entry replacement)
-           (wrap-external-format-functions
-              entry
-              (lambda (fun)
-                (and fun
-                     (lambda (&rest rest)
-                       (declare (dynamic-extent rest))
-                       (handler-bind
-                           ((stream-decoding-error
-                             (lambda (c)
-                               (declare (ignore c))
-                               (invoke-restart 'input-replacement replacement)))
-                            (stream-encoding-error
-                             (lambda (c)
-                               (declare (ignore c))
-                               (invoke-restart 'output-replacement replacement)))
-                            (octets-encoding-error
-                             (lambda (c) (use-value replacement c)))
-                            (octet-decoding-error
-                             (lambda (c) (use-value replacement c))))
-                         (apply fun rest))))))))
-
+  (let* ((external-format (ensure-list external-format))
+         (options (cdr external-format)))
+    (unless (symbolp (car external-format))
+      (return-from get-external-format nil))
+    (loop for (option value) on options by 'cddr
+          unless (or (eql option :newline) (eql option :replacement))
+          do (return-from get-external-format nil))
     (binding*
-        (((format-name replacement)
-          (etypecase external-format
-            ;; This seem like such an unnecessarily general way to
-            ;; pass an optional parameter. What's up with that???
-            ((cons keyword)
-             (values (car external-format)
-                     (getf (cdr external-format) :replacement)))
-            (symbol
-             (values external-format nil))))
+        (((format-name newline replacement)
+          (values (car external-format)
+                  (getf options :newline :lf)
+                  (getf options :replacement)))
          (table-index (get format-name :external-format) :exit-if-null)
          (formats *external-formats*)
          (table-entry
           ;; The table entry can be one of:
           ;;   1. #<external-format>
-          ;;   2. (#<external-format> (#\char . #<modified-ef>) ...)
+          ;;   2. (#<external-format> ((:crlf #\char) . #<modified-ef>) ...)
           ;;      for a list of modified formats that alter the
-          ;;      choice of replacement character.
+          ;;      newline encoding and choice of replacement character.
           ;;   3. "namestring" - to autoload from a fasl containing
           ;;      the named format.
           (let ((ef (svref formats table-index)))
@@ -501,22 +504,32 @@ STRING (or the subsequence bounded by START and END)."
          ((base-format variations)
           (if (listp table-entry)
               (values (car table-entry) (cdr table-entry))
-              (values table-entry nil))))
-      (when (or (not base-format) (not replacement))
-        (return-from get-external-format base-format))
+              (values table-entry nil)))
+         (newline-base-format
+          (if (eql newline :lf)
+              base-format
+              (cdr (assoc (list newline) variations :test #'equal)))
+          :exit-if-null))
+      (unless (typep replacement '(or null character string (unsigned-byte 8) (simple-array (unsigned-byte 8) 1)))
+        (return-from get-external-format nil))
+      (when (or (not newline-base-format) (not replacement))
+        (return-from get-external-format newline-base-format))
       (loop
-        (awhen (assoc replacement variations)
-          (return (cdr it)))
-        (let* ((new-ef (replacement-handlerify base-format replacement))
-               (new-table-entry
-                (cons base-format (acons replacement new-ef variations)))
-               (old (cas (svref formats table-index) table-entry new-table-entry)))
-          (when (eq old table-entry)
-            (return new-ef))
-          ;; CAS failure -> some other thread added an entry. It's probably
-          ;; for the same replacement char which is usually #\ufffd.
-          ;; So try again. At worst this conses some more garbage.
-          (setq table-entry old))))))
+        (let ((key (cons newline replacement)))
+          (awhen (assoc key variations :test #'equal)
+            (return (cdr it)))
+          (let* ((new-ef (let ((copy (copy-structure newline-base-format)))
+                           (setf (ef-replacement copy) replacement)
+                           copy))
+                 (new-table-entry
+                  (cons base-format (acons key new-ef variations)))
+                 (old (cas (svref formats table-index) table-entry new-table-entry)))
+            (when (eq old table-entry)
+              (return new-ef))
+            ;; CAS failure -> some other thread added an entry. It's probably
+            ;; for the same replacement char which is usually #\ufffd.
+            ;; So try again. At worst this conses some more garbage.
+            (setq table-entry old)))))))
 
 (push
   `("SB-IMPL"
