@@ -54,7 +54,9 @@
   "Return T if X is NIL, otherwise return NIL."
   (not object))
 
-;;; All the primitive type predicate wrappers share a parallel form..
+;;; All the primitive type predicate wrappers share a parallel form.
+;;; These aren't so much "wrappers" as they are the actual installed DEFUNs.
+;;; I supposed they "wrap" a vop or source-transform.
 (macrolet ((def-type-predicate-wrapper (pred)
              (let* ((name (symbol-name pred))
                     (stem (string-left-trim "%" (string-right-trim "P-" name)))
@@ -106,13 +108,12 @@
   (def-type-predicate-wrapper integerp)
   (def-type-predicate-wrapper listp)
   (def-type-predicate-wrapper long-float-p)
-  #-(or x86 x86-64 arm64) (def-type-predicate-wrapper lra-p)
+  #-(or x86 x86-64 arm64 riscv) (def-type-predicate-wrapper lra-p)
   (def-type-predicate-wrapper null)
   (def-type-predicate-wrapper numberp)
   (def-type-predicate-wrapper rationalp)
   (def-type-predicate-wrapper ratiop)
   (def-type-predicate-wrapper realp)
-  (def-type-predicate-wrapper short-float-p)
   (def-type-predicate-wrapper single-float-p)
   #+sb-simd-pack (def-type-predicate-wrapper simd-pack-p)
   #+sb-simd-pack-256 (def-type-predicate-wrapper simd-pack-256-p)
@@ -126,10 +127,15 @@
   (def-type-predicate-wrapper system-area-pointer-p)
   (def-type-predicate-wrapper unbound-marker-p)
   (def-type-predicate-wrapper weak-pointer-p)
-  #-64-bit
-  (progn
-    (def-type-predicate-wrapper unsigned-byte-32-p)
+
+  (sb-c::when-vop-existsp (:translate signed-byte-8-p)
+    (def-type-predicate-wrapper signed-byte-8-p))
+  (sb-c::when-vop-existsp (:translate signed-byte-16-p)
+    (def-type-predicate-wrapper signed-byte-16-p))
+  (sb-c::when-vop-existsp (:translate signed-byte-32-p)
     (def-type-predicate-wrapper signed-byte-32-p))
+  #-64-bit
+  (def-type-predicate-wrapper unsigned-byte-32-p)
   #+64-bit
   (progn
     (def-type-predicate-wrapper unsigned-byte-64-p)
@@ -150,10 +156,14 @@
   (def-type-predicate-wrapper stringp)
   (def-type-predicate-wrapper vectorp))
 
-#+(or x86 x86-64 arm arm64)
-(defun fixnum-mod-p (x limit)
-  (and (fixnump x)
-       (<= 0 x limit)))
+(sb-c::when-vop-existsp (:translate fixnum-mod-p)
+  (defun fixnum-mod-p (x limit)
+    (and (fixnump x)
+         (<= 0 x limit))))
+
+(sb-c::when-vop-existsp (:translate car-eq-if-listp)
+  (defun car-eq-if-listp (value object)
+    (car-eq-if-listp value object)))
 
 
 ;;; Return the specifier for the type of object. This is not simply
@@ -203,19 +213,11 @@
      (type-specifier (ctype-of object)))
     (simple-fun 'compiled-function)
     (t
-     #+metaspace ; WRAPPER-OF can't be called on layoutless objects.
-     (unless (logtest (get-lisp-obj-address (%instanceoid-layout object))
-                      sb-vm:widetag-mask)
-       ;; [fun-]instances momentarily have no layout in any code interrupted
-       ;; just after allocating and before assigning slots.
-       ;; OUTPUT-UGLY-OBJECT has a similar precaution as this.
-       (return-from type-of (if (functionp object) 'funcallable-instance 'instance)))
-     (let ((wrapper (wrapper-of object)))
-       #-metaspace ; already checked for a good layout if metaspace
-       (when (= (get-lisp-obj-address wrapper) 0)
+     (let ((layout (layout-of object)))
+       (when (= (get-lisp-obj-address layout) 0)
          (return-from type-of
            (if (functionp object) 'funcallable-instance 'instance)))
-       (let* ((classoid (wrapper-classoid wrapper))
+       (let* ((classoid (layout-classoid layout))
               (name (classoid-name classoid)))
          ;; FIXME: should the first test be (not (or (%instancep) (%funcallable-instance-p)))?
          ;; God forbid anyone makes anonymous classes of generic functions.
@@ -236,20 +238,6 @@
 (defun eq (obj1 obj2)
   "Return T if OBJ1 and OBJ2 are the same object, otherwise NIL."
   (eq obj1 obj2))
-;;; and this too, but it's only needed for backends on which
-;;; IR1 might potentially transform EQL into %EQL/INTEGER.
-#+integer-eql-vop
-(defun %eql/integer (obj1 obj2)
-  ;; This is just for constant folding, there's no real need to transform
-  ;; into the %EQL/INTEGER VOP. But it's confusing if it becomes identical to
-  ;; EQL, and then due to ICF we find that #'EQL => #<FUNCTION %EQL/INTEGER>.
-  ;; Type declarations don't suffice because we don't know which arg is an integer
-  ;; (if not both). We could ensure selection of the vop by using %PRIMITIVE.
-  ;; Anyway it suffices to disable type checking and pretend its the always
-  ;; the first arg that's an integer, but that won't work on the host because
-  ;; it might enforce the type since we can't portably unenforce after declaring.
-  (declare (optimize (sb-c::type-check 0)))
-  (eql (the integer obj1) obj2))
 
 (declaim (inline %eql))
 (defun %eql (obj1 obj2)
@@ -275,8 +263,10 @@
              #+long-float
              (long-float eql)
              (bignum
-              #-integer-eql-vop (lambda (x y) (zerop (bignum-compare x y)))
-              #+integer-eql-vop eql) ; will become %eql/integer
+              #.(sb-c::if-vop-existsp (:named sb-vm::%eql/integer)
+                  'eql
+                  '(lambda (x y)
+                    (zerop (bignum-compare x y)))))
              (ratio
               (lambda (x y)
                 (and (eql (numerator x) (numerator y))
@@ -508,8 +498,7 @@ length and have identical components. Other arrays must be EQ to be EQUAL."
                 (and (logtest (logior +structure-layout-flag+ +pathname-layout-flag+)
                               (layout-flags layout))
                      (eq (%instance-layout y) layout)
-                     (funcall (wrapper-equalp-impl (layout-friend layout))
-                              x y)))))
+                     (funcall (layout-equalp-impl layout) x y)))))
         ((arrayp x)
          (and (arrayp y)
               ;; string-equal is nearly 2x the speed of array-equalp for comparing strings
@@ -518,6 +507,7 @@ length and have identical components. Other arrays must be EQ to be EQUAL."
                     (t (array-equalp x y)))))
         (t nil)))
 
+#-sb-show ;; I don't know why these tests crash with #+sb-show
 (let ((test-cases `(($0.0 $-0.0 t)
                     ($0.0 $1.0 nil)
                     ;; There is no cross-compiler #C reader macro.
