@@ -864,7 +864,7 @@ The result is not guaranteed to have the same length as the input."
                      ((>= end length)
                       (if (= end length) (progn (funcall function string start end) nil)))
                   (flet ((brk () (funcall function string start end) (setf start end)))
-                    (declare (truly-dynamic-extent #'brk))
+                    (declare (dynamic-extent #'brk))
                     (shiftf char1 char2 (char string end))
                     (shiftf c1 c2 (grapheme-break-class char2))
                     (if (eql c2 :regional-indicator) (incf nri) (setf nri 0))
@@ -903,7 +903,7 @@ The result is not guaranteed to have the same length as the input."
                   (flet ((fun (string start end)
                            (declare (type string string))
                            (funcall function (adjust-array array (- end start) :displaced-to string :displaced-index-offset start))))
-                    (declare (truly-dynamic-extent #'fun))
+                    (declare (dynamic-extent #'fun))
                     (,mapper #'fun string))))))
   (def map-legacy-graphemes map-legacy-grapheme-boundaries)
   (def map-graphemes map-grapheme-boundaries))
