@@ -364,26 +364,53 @@
           (loadw value tmp-tn ,offset ,lowtag))))))
 
 (defmacro define-full-setter (name type offset lowtag scs el-type
-                              &optional translate)
-  `(define-vop (,name)
-     ,@(when translate
-         `((:translate ,translate)))
-     (:policy :fast-safe)
-     (:args (object :scs (descriptor-reg))
-            (index :scs (any-reg unsigned-reg signed-reg immediate))
-            (value :scs (,@scs zero)))
-     (:arg-types ,type tagged-num ,el-type)
-     (:generator 2
-       (sc-case index
-         (immediate
-          (inst str value (@ object (load-store-offset
-                                     (- (ash (+ ,offset (tn-value index)) word-shift)
-                                        ,lowtag)))))
-         (t
-          (inst add tmp-tn object (lsl index (- word-shift (if (sc-is index any-reg)
-                                                               n-fixnum-tag-bits
-                                                               0))))
-          (storew value tmp-tn ,offset ,lowtag))))))
+                              &optional translate
+                              &aux (barrierp (member 'descriptor-reg scs)))
+  (case name
+    ((data-vector-set/simple-vector %weakvec-set)
+     `(define-vop (,name)
+        (:translate ,translate)
+        (:policy :fast-safe)
+        (:args (object :scs (descriptor-reg))
+               (index :scs (any-reg unsigned-reg signed-reg immediate))
+               (value :scs (,@scs zero)))
+        (:arg-types ,type tagged-num ,el-type)
+        (:temporary (:sc non-descriptor-reg) ea)
+        (:vop-var vop)
+        (:generator 2
+          (sc-case index
+            (immediate
+             (inst add ea object (load-store-offset
+                                  (- (ash (+ ,offset (tn-value index)) word-shift)
+                                     ,lowtag))))
+            (t
+             ;; Scale the index
+             (let ((unshift (if (sc-is index any-reg) n-fixnum-tag-bits 0)))
+               (inst add ea object (lsl index (- word-shift unshift))))
+             ;; Calculate the exact cell address to ensure the right card is marked
+             (inst add ea ea (- (ash vector-data-offset word-shift) other-pointer-lowtag))))
+          (emit-gengc-barrier object ea tmp-tn (vop-nth-arg 2 vop) value)
+          (storew value ea 0 0))))
+    (t
+     `(define-vop (,name)
+        (:translate ,translate)
+        (:policy :fast-safe)
+        (:args (object :scs (descriptor-reg))
+               (index :scs (any-reg unsigned-reg signed-reg immediate))
+               (value :scs (,@scs zero)))
+        (:arg-types ,type tagged-num ,el-type)
+        (:vop-var vop)
+        (:generator 2
+          ,@(if barrierp '((emit-gengc-barrier object nil tmp-tn (vop-nth-arg 2 vop) value)))
+          (sc-case index
+            (immediate
+             (inst str value (@ object (load-store-offset
+                                        (- (ash (+ ,offset (tn-value index)) word-shift)
+                                           ,lowtag)))))
+            (t
+             (let ((unshift (if (sc-is index any-reg) n-fixnum-tag-bits 0)))
+               (inst add tmp-tn object (lsl index (- word-shift unshift))))
+             (storew value tmp-tn ,offset ,lowtag))))))))
 
 (defmacro define-partial-reffer (name type size signed offset lowtag scs
                                  el-type &optional translate)
