@@ -83,7 +83,7 @@
                 packagep functionp compiled-function-p not)
   (t) boolean (movable foldable flushable))
 
-(defknown (eq eql %eql/integer) (t t) boolean
+(defknown (eq eql) (t t) boolean
   (movable foldable flushable commutative))
 (defknown (equal equalp) (t t) boolean (foldable flushable recursive))
 
@@ -597,11 +597,14 @@
     vector
   (flushable no-verify-arg-count))
 
+(defknown (possibly-base-stringize possibly-base-stringize-to-heap) (string) simple-string
+  (flushable no-verify-arg-count))
+
 (defknown map (type-specifier (function-designator ((nth-arg 2 :sequence t)
                                                     (rest-args :sequence t))
                                                    (nth-arg 0 :sequence-type t))
                               proper-sequence &rest proper-sequence)
-  consed-sequence (call))
+    consed-sequence (call))
 (defknown %map (type-specifier function-designator &rest sequence) consed-sequence
   (call no-verify-arg-count))
 (defknown %map-for-effect-arity-1 (function-designator sequence) null
@@ -636,8 +639,8 @@
 (defknown (every notany notevery) (function-designator proper-sequence &rest proper-sequence) boolean
   (foldable unsafely-flushable call))
 
-(defknown reduce ((function-designator ((nth-arg 1 :sequence t :key :key :value :initial-value)
-                                        (nth-arg 1 :sequence t :key :key)))
+(defknown reduce ((function-designator ((nth-arg 1 :sequence t :key :key :value (:initial-value :from-end nil))
+                                        (nth-arg 1 :sequence t :key :key :value (:initial-value :from-end t))))
                   proper-sequence &rest t &key (:from-end t)
                   (:start (inhibit-flushing index 0))
                   (:end (inhibit-flushing sequence-end nil))
@@ -1726,6 +1729,8 @@
 
 (defknown (signal warn) (condition-designator-head &rest t) null)
 (defknown error (condition-designator-head &rest t) nil)
+(defknown %simple-error (condition-designator-head t &rest t) nil)
+(defknown %simple-type-error (condition-designator-head t t t &rest t) nil)
 (defknown cerror (format-control condition-designator-head &rest t) null)
 (defknown invalid-method-error (t format-control &rest t) *) ; FIXME: first arg is METHOD
 (defknown method-combination-error (format-control &rest t) *)
@@ -1862,6 +1867,23 @@
 
 (defknown array-storage-vector (array) (simple-array * (*))
     (any))
+
+(defknown octets-to-string ((vector (unsigned-byte 8))
+                            &key
+                            (:external-format t)
+                            (:start index)
+                            (:end sequence-end))
+  (or (simple-array character (*)) (simple-array base-char (*)))
+  (flushable))
+(defknown string-to-octets (string
+                            &key
+                            (:external-format t)
+                            (:start index)
+                            (:end sequence-end)
+                            (:null-terminate t))
+  (simple-array (unsigned-byte 8) (*))
+  (flushable))
+
 
 ;;;; magical compiler frobs
 
@@ -1894,16 +1916,12 @@
   :derive-type #'result-type-first-arg)
 
 (defknown %cleanup-point (&rest t) t (reoptimize-when-unlinking))
-(defknown %dynamic-extent-start (t) t)
+(defknown %dynamic-extent-start () t)
 (defknown %special-bind (t t) t)
 (defknown %special-unbind (&rest symbol) t)
 (defknown %listify-rest-args (t index) list (flushable))
 (defknown %more-arg-context (t t) (values t index) (flushable))
 (defknown %more-arg (t index) t (flushable))
-(defknown %more-keyword-pair (t fixnum) (values t t) (flushable))
-#+stack-grows-downward-not-upward
-;;; FIXME: The second argument here should really be NEGATIVE-INDEX, but doing that
-;;; breaks the build, and I cannot seem to figure out why. --NS 2006-06-29
 (defknown %more-kw-arg (t fixnum) (values t t))
 (defknown %more-arg-values (t index index) * (flushable))
 
@@ -1939,7 +1957,9 @@
   (movable foldable flushable no-verify-arg-count))
 (defknown %negate (number) number
   (movable foldable flushable no-verify-arg-count))
-(defknown (%check-bound check-bound) (array index t) index
+(defknown check-bound (array index t) index
+  (dx-safe))
+(defknown %check-bound (array index t) (values)
   (dx-safe))
 (defknown data-vector-ref (simple-array index) t
   (foldable flushable always-translatable))
@@ -1973,7 +1993,7 @@
 (defknown %set-symbol-package (symbol t) t ())
 (defknown (%coerce-callable-to-fun %coerce-callable-for-call)
     (function-designator)
-    function (flushable))
+    function (flushable no-verify-arg-count))
 (defknown array-bounding-indices-bad-error (t t t) nil)
 (defknown sequence-bounding-indices-bad-error (t t t) nil)
 (defknown %find-position
