@@ -95,28 +95,6 @@
      (- other-pointer-lowtag)
      (* fdefn-raw-addr-slot n-word-bytes)))
 
-;;; Various error-code generating helpers
-(defvar *adjustable-vectors*)
-
-(defmacro with-adjustable-vector ((var) &rest body)
-  `(let ((,var (or (pop *adjustable-vectors*)
-                   (make-array 16
-                               :element-type '(unsigned-byte 8)
-                               :fill-pointer 0
-                               :adjustable t))))
-     ;; Don't declare the length - if it gets adjusted and pushed back
-     ;; onto the freelist, it's anyone's guess whether it was expanded.
-     ;; This code was wrong for >12 years, so nobody must have needed
-     ;; more than 16 elements. Maybe we should make it nonadjustable?
-     (declare (type (vector (unsigned-byte 8)) ,var))
-     (setf (fill-pointer ,var) 0)
-     ;; No UNWIND-PROTECT here - semantics are unaffected by nonlocal exit,
-     ;; and this macro is about speeding up the compiler, not slowing it down.
-     ;; GC will clean up any debris, and since the vector does not point
-     ;; to anything, even an accidental promotion to a higher generation
-     ;; will not cause transitive garbage retention.
-     (prog1 (progn ,@body)
-       (push ,var *adjustable-vectors*))))
 
 ;;;; interfaces to IR2 conversion
 
@@ -159,7 +137,9 @@
           #-c-stack-is-control-stack *non-descriptor-args*))
 
 (defun fixed-call-arg-location (type state)
-  (let* ((primtype (primitive-type type))
+  (let* ((primtype (if (typep type 'primitive-type)
+                       type
+                       (primitive-type type)))
          (sc (find descriptor-reg-sc-number (sb-c::primitive-type-scs primtype) :test-not #'eql)))
     (case (primitive-type-name primtype)
       ((double-float single-float)
@@ -181,10 +161,8 @@
 
 ;;; Make a TN to hold the number-stack frame pointer.  This is allocated
 ;;; once per component, and is component-live.
-(defun make-nfp-tn ()
-  #+c-stack-is-control-stack
-  (make-restricted-tn *fixnum-primitive-type* ignore-me-sc-number)
   #-c-stack-is-control-stack
+(defun make-nfp-tn ()
   (component-live-tn
    (make-wired-tn *fixnum-primitive-type* immediate-arg-scn nfp-offset)))
 
@@ -201,7 +179,7 @@
 
 (defun make-number-stack-pointer-tn ()
   #+c-stack-is-control-stack
-  (make-restricted-tn *fixnum-primitive-type* ignore-me-sc-number)
+  (sb-c::make-unused-tn)
   #-c-stack-is-control-stack
   (make-normal-tn *fixnum-primitive-type*))
 
@@ -230,7 +208,9 @@
   "Cause a continuable error.  ERROR-CODE is the error to cause."
   (emit-error-break vop cerror-trap (error-number-or-lose error-code) values))
 
-#+sb-safepoint
+;; C header generation can't execute DEFINE-VOP
+;; Perhaps this belongs somewhere different
+#+(and sb-safepoint (not :c-headers-only))
 (define-vop (insert-safepoint)
   (:policy :fast-safe)
   (:translate sb-kernel::gc-safepoint)
@@ -257,39 +237,75 @@
 ;;; then NIL is ok as the input. Indicate this by specifying PERMIT-NIL.
 ;;; With rare exception it should always be permitted, though not on ppc64
 ;;; where it would never be. The safe default is NIL.
-(defun other-pointer-tn-ref-p (tn-ref &optional permit-nil)
+(defun other-pointer-tn-ref-p (tn-ref &optional permit-nil
+                                                immediates-tested)
   (and (sc-is (tn-ref-tn tn-ref) descriptor-reg)
        (not (types-equal-or-intersect
              (tn-ref-type tn-ref)
              (if permit-nil
-                 (specifier-type '(or fixnum
-                                   #+64-bit single-float
-                                   function
-                                   cons
-                                   instance
-                                   character))
-                 (specifier-type '(or fixnum
-                                   #+64-bit single-float
-                                   function
-                                   list
-                                   instance
-                                   character)))))))
+                 (specifier-type '(or function cons instance character))
+                 (specifier-type '(or function list instance character)))))
+       (or (memq 'fixnum immediates-tested)
+           (not (types-equal-or-intersect (tn-ref-type tn-ref) (specifier-type 'fixnum))))
+       #+64-bit
+       (or (memq single-float-widetag immediates-tested)
+           (not (types-equal-or-intersect (tn-ref-type tn-ref) (specifier-type 'single-float))))))
 
 (defun fixnum-or-other-pointer-tn-ref-p (tn-ref &optional permit-nil)
   (and (sc-is (tn-ref-tn tn-ref) descriptor-reg)
        (not (types-equal-or-intersect
              (tn-ref-type tn-ref)
-             (specifier-type (if permit-nil
-                                 '(or #+64-bit single-float
-                                   function
-                                   cons
-                                   instance
-                                   character)
-                                 '(or #+64-bit single-float
-                                   function
-                                   list
-                                   instance
-                                   character)))))))
+             (if permit-nil
+                 (specifier-type '(or cons . #1=(#+64-bit single-float function cons instance character)))
+                 (specifier-type '(or list . #1#)))))))
+
+;;; Can LOWTAG be distinguished from other tn lowtags by testing a single bit?
+(defun tn-ref-lowtag-bit (lowtag tn-ref &optional permit-nil immediates-tested)
+  (declare (fixnum lowtag))
+  (when tn-ref
+    (let ((type (tn-ref-type tn-ref))
+          (set 0)
+          (clear 0))
+      (when (eq type *universal-type*)
+        (return-from tn-ref-lowtag-bit))
+      (unless (memq 'fixnum immediates-tested)
+        (when (types-equal-or-intersect type (specifier-type 'fixnum))
+          (setf set (logandc2 lowtag-mask fixnum-tag-mask)
+                clear lowtag-mask)))
+      (macrolet ((s (lowtag type)
+                   `(when (and (/= lowtag ,lowtag)
+                               (types-equal-or-intersect type ,(if (symbolp type)
+                                                                   `(specifier-type ',type)
+                                                                   type)))
+                      (setf set (logior set (logand lowtag-mask ,lowtag))
+                            clear (logior clear (logandc2 lowtag-mask ,lowtag))))))
+        #+64-bit
+        (unless (memq single-float-widetag immediates-tested)
+          (s single-float-widetag single-float))
+        (if permit-nil
+            (s list-pointer-lowtag cons)
+            (s list-pointer-lowtag list))
+        (s fun-pointer-lowtag function)
+        (s instance-pointer-lowtag instance)
+        (s character-widetag character)
+        (s other-pointer-lowtag
+           (if permit-nil
+               (specifier-type '(not (or cons . #1=(#+64-bit fixnum single-float function cons instance character))))
+               (specifier-type '(not (or list . #1#)))))
+        (let ((set-bit (logand lowtag-mask (logandc2 lowtag set)))
+              (clear-bit (logandc2 lowtag-mask (logior lowtag clear))))
+          (cond ((plusp set-bit)
+                 (values (sb-kernel::first-bit-set set-bit) 1))
+                ((plusp clear-bit)
+                 (values (sb-kernel::first-bit-set clear-bit) 0))))))))
+
+(defun fun-or-other-pointer-tn-ref-p (tn-ref &optional permit-nil)
+  (and (sc-is (tn-ref-tn tn-ref) descriptor-reg)
+       (not (types-equal-or-intersect
+             (tn-ref-type tn-ref)
+             (if permit-nil
+                 (specifier-type '(or cons instance character fixnum #+64-bit single-float))
+                 (specifier-type '(or cons instance character fixnum #+64-bit single-float)))))))
 
 (defun not-nil-tn-ref-p (tn-ref)
   (not (types-equal-or-intersect (tn-ref-type tn-ref)
@@ -298,6 +314,12 @@
 (defun instance-tn-ref-p (tn-ref)
   (csubtypep (tn-ref-type tn-ref) (specifier-type 'instance)))
 
+;;; Note that this is a allowed to fail by returning NIL.
+;;; So it's really testing "CERTAINLY-STACK-CONSED-P", which is
+;;; T if and only if if knows, and NIL if it doesn't know,
+;;; or OBJECT is heap-consed.
+;;; In general this is a crummy way to deduce the object's creator,
+;;; because MOVE-OPERAND has a nasty way of interfering.
 (defun stack-consed-p (object)
   (let ((write (sb-c::tn-writes object))) ; list of write refs
     (when (or (not write)    ; grrrr, the only write is from a LOAD tn
@@ -332,11 +354,11 @@
 (define-load-time-global *store-barriers-potentially-emitted* 0)
 (define-load-time-global *store-barriers-emitted* 0)
 
-(defun require-gc-store-barrier-p (object value-tn-ref value-tn)
+(defun require-gengc-barrier-p (object value-tn-ref value-tn)
   (incf *store-barriers-potentially-emitted*)
   ;; If OBJECT is stack-allocated, elide the barrier
   (when (stack-consed-p object)
-    (return-from require-gc-store-barrier-p nil))
+    (return-from require-gengc-barrier-p nil))
   (flet ((potential-heap-pointer-p (tn tn-ref)
            (when (sc-is tn any-reg) ; must be fixnum
              (return-from potential-heap-pointer-p nil))
@@ -361,7 +383,7 @@
            (unless (eq (tn-ref-tn value-tn-ref) value-tn)
              (aver (eq (tn-ref-load-tn value-tn-ref) value-tn)))
            (unless (potential-heap-pointer-p value-tn value-tn-ref)
-             (return-from require-gc-store-barrier-p nil)))
+             (return-from require-gengc-barrier-p nil)))
           (value-tn-ref ; a list of refs linked through TN-REF-ACROSS
            ;; (presumably from INSTANCE-SET-MULTIPLE)
            (let ((any-pointer
@@ -370,7 +392,7 @@
                     (when (potential-heap-pointer-p (tn-ref-tn ref) ref)
                       (return t)))))
              (unless any-pointer
-               (return-from require-gc-store-barrier-p nil))))))
+               (return-from require-gengc-barrier-p nil))))))
   (incf *store-barriers-emitted*)
   t)
 
@@ -464,3 +486,35 @@
                     smallest-f d n))
            (values (ceiling (expt 2 fraction-bits) d) fraction-bits)))))
 
+(defun system-tlab-p (type node)
+  #-system-tlabs (declare (ignore type node))
+  #+system-tlabs
+  (or sb-c::*force-system-tlab*
+      (let ((typename (cond ((sb-kernel::layout-p type)
+                             (classoid-name (layout-classoid type)))
+                            ((sb-kernel::defstruct-description-p type)
+                             (dd-name type)))))
+        (when (and typename (sb-xc:subtypep typename 'ctype))
+          (error "~S instance constructor called in a non-system file"
+                 typename)))
+      (and node
+           (named-let search-env ((env (sb-c::node-lexenv node)))
+             (dolist (data (sb-c::lexenv-user-data env)
+                           (and (sb-c::lexenv-parent env)
+                                (search-env (sb-c::lexenv-parent env))))
+               (when (and (eq (first data) :declare)
+                          (eq (second data) 'sb-c::tlab))
+                 (return (eq (third data) :system))))))))
+
+(defun call-out-pseudo-atomic-p (vop)
+  (declare (ignorable vop))
+  ;; If #+sb-safepoint, the decision to poll for a safepoint
+  ;; occurs at the end. In that case, we can not prevent stop-for-GC
+  ;; from occurring in the C code, because foreign code is allowed
+  ;; to run during GC; it just can't go back into Lisp until GC is over.
+  #-sb-safepoint
+  (loop for e = (sb-c::node-lexenv (sb-c::vop-node vop))
+        then (sb-c::lexenv-parent e)
+        while e
+        thereis (sb-c::lexenv-find 'sb-vm::.pseudo-atomic-call-out.
+                                   vars :lexenv e)))
