@@ -194,8 +194,8 @@
   ;; the optimizer for this node type doesn't care, it can elect not
   ;; to clear this flag.
   (reoptimize t :type boolean)
-  ;; if the LVAR value is DYNAMIC-EXTENT, some information.
-  (dynamic-extent nil :type (or null dx-info))
+  ;; the DYNAMIC-EXTENT of this lvar
+  (dynamic-extent nil :type (or null cdynamic-extent))
   ;; something or other that the back end annotates this lvar with
   (info nil)
   ;; Nodes to reoptimize together with the lvar
@@ -204,25 +204,6 @@
   (dependent-annotations nil))
 (!set-load-form-method lvar (:xc :target) :ignore-it)
 
-;;; A DX-INFO structure is used to accumulate information about a
-;;; dynamic extent declaration.
-(defstruct (dx-info (:copier nil))
-  ;; The kind of dynamic extent this is.
-  (kind (missing-arg) :type (member enclose dynamic-extent truly-dynamic-extent))
-  ;; The value recognized to be declared dynamic extent.
-  (value (missing-arg) :type lvar)
-  ;; The stack-allocatable values in the transitive closure of the
-  ;; relation determined by the "otherwise inaccessible part"
-  ;; criterion. This is filled in by environment analysis.
-  (subparts nil :type list)
-  ;; The CLEANUP associated with this dynamic extent.
-  (cleanup (missing-arg) :type cleanup))
-
-(defprinter (dx-info :identity t)
-  kind
-  value
-  subparts)
-
 ;;; These are used for annotating a LVAR with information that can't
 ;;; be expressed using types or if the CAST semantics are undesirable
 ;;; (type derivation, runtime errors).
@@ -365,14 +346,10 @@
 (defun node-block (node)
   (ctran-block (node-prev node)))
 
-(defun %with-ir1-environment-from-node (node fun)
-  (declare (type node node) (type function fun))
-  (declare (dynamic-extent fun))
-  (let ((*current-component* (node-component node))
-        (*lexenv* (node-lexenv node))
-        (*current-path* (node-source-path node)))
-    (aver-live-component *current-component*)
-    (funcall fun)))
+(declaim (inline node-component))
+(defun node-component (node)
+  (declare (type node node))
+  (block-component (node-block node)))
 
 (defstruct (valued-node (:conc-name node-)
                         (:include node)
@@ -656,8 +633,6 @@
   ;; from COMPONENT-LAMBDAS.
   (reanalyze-functionals nil :type list)
   (delete-blocks nil :type list)
-  ;; this is filled by environment analysis
-  (dx-lvars nil :type list)
   ;; The default LOOP in the component.
   (outer-loop (make-loop :kind :outer :head head :tail (list tail)) :type cloop)
   (max-block-number 0 :type fixnum)
@@ -682,7 +657,6 @@
   ;; COMPILE-COMPONENT hasn't happened yet. Might it be even better
   ;; (certainly stricter, possibly also correct...) to assert that
   ;; IR1-FINALIZE hasn't happened yet?
-  #+sb-xc-host (declare (notinline component-info)) ; unknown type
   (aver (not (eql (component-info component) :dead))))
 
 ;;; A CLEANUP structure represents some dynamic binding action. Blocks
@@ -707,12 +681,8 @@
   ;; non-messed-up environment. Null only temporarily. This could be
   ;; deleted due to unreachability.
   (mess-up nil :type (or node null))
-  ;; For all kinds, except :DYNAMIC-EXTENT: a list of all the NLX-INFO
-  ;; structures whose NLX-INFO-CLEANUP is this cleanup. This is filled
-  ;; in by environment analysis.
-  ;;
-  ;; For :DYNAMIC-EXTENT: a list of all DX-INFOs, preserved by this
-  ;; cleanup.
+  ;; a list of all the NLX-INFO structures whose NLX-INFO-CLEANUP is
+  ;; this cleanup.
   (nlx-info nil :type list))
 (defprinter (cleanup :identity t)
   kind
@@ -723,13 +693,8 @@
 (defstruct (environment (:copier nil))
   ;; the function that allocates this environment
   (lambda (missing-arg) :type clambda :read-only t)
-  ;; This ultimately converges to a list of all the LAMBDA-VARs and
-  ;; NLX-INFOs needed from enclosing environments by code in this
-  ;; environment. In the meantime, it may be
-  ;;   * NIL at object creation time
-  ;;   * a superset of the correct result, generated somewhat later
-  ;;   * smaller and smaller sets converging to the correct result as
-  ;;     we notice and delete unused elements in the superset
+  ;; a list of all the LAMBDA-VARs and NLX-INFOs needed from enclosing
+  ;; environments by code in this environment.
   (closure nil :type list)
   ;; a list of NLX-INFO structures describing all the non-local exits
   ;; into this environment
@@ -865,8 +830,8 @@
   ;; T if there was a REF.
   ;; This may be non-nil when REFS and SETS are null, since code can be deleted.
   (ever-used nil :type (member nil set t))
-  ;; is it declared dynamic-extent, or truly-dynamic-extent?
-  (dynamic-extent nil :type (member nil truly-dynamic-extent dynamic-extent))
+  ;; True if declared dynamic-extent.
+  (dynamic-extent nil :type boolean)
   ;; some kind of info used by the back end
   (info nil))
 (!set-load-form-method leaf (:xc :target) :ignore-it)
@@ -1213,6 +1178,9 @@
   (lets nil :type list)
   ;; all the ENTRY nodes in this function and its LETs, or null in a LET
   (entries nil :type list)
+  ;; all the DYNAMIC-EXTENT nodes in this function and its LETs, or
+  ;; null in a LET.
+  (dynamic-extents nil :type list)
   ;; CLAMBDAs which are locally called by this lambda, and other
   ;; objects (closed-over LAMBDA-VARs and XEPs) which this lambda
   ;; depends on in such a way that DFO shouldn't put them in separate
@@ -1465,9 +1433,7 @@
   ;; The leaf referenced.
   (leaf nil :type leaf)
   ;; KLUDGE: This is supposed to help with keyword debug messages somehow.
-  (%source-name (missing-arg) :type symbol :read-only t)
-  ;; Constraints that cannot be expressed as NODE-DERIVED-TYPE
-  (constraints nil))
+  (%source-name (missing-arg) :type symbol :read-only t))
 (defprinter (ref :identity t)
   (%source-name :test (neq %source-name '.anonymous.))
   leaf)
@@ -1640,10 +1606,9 @@
   type-to-check)
 
 ;;; The DELAY node is interposed between a VALUE's USE and its DEST in
-;;; order to allow the value to be immediately used. This is necessary
-;;; for implementing multiple-use unknown values LVARs, as otherwise,
-;;; a non-moveable dynamic extent object may be allocated between the
-;;; DEST and one of the LVAR's uses but not the others.
+;;; order to allow the value to be immediately used. This allows us to
+;;; do substitution of lvars without doing flow analysis to check the
+;;; validity of the substitution in certain cases.
 (defstruct (delay (:include cast
                    (%type-check nil)
                    (asserted-type *wild-type*)
@@ -1709,16 +1674,38 @@
 
 ;;; The ENCLOSE node marks the place at which closure allocation code
 ;;; would be emitted, if necessary.
-(defstruct (enclose (:include valued-node) ; this node uses a dummy lvar for dx analysis
+(defstruct (enclose (:include node)
                     (:copier nil))
   ;; The list of functionals that this ENCLOSE node allocates.
   (funs nil :type list)
-  ;; The cleanup for this enclose if any of its functionals are
+  ;; The dynamic extent for this enclose if any of its functionals are
   ;; declared dynamic extent.
-  (cleanup nil :type (or null cleanup)))
+  (dynamic-extent nil :type (or null cdynamic-extent)))
 (defprinter (enclose :identity t)
   funs)
 
+;;; The DYNAMIC-EXTENT node is used to accumulate information about a
+;;; dynamic extent declaration. It is the mess-up for the
+;;; corresponding :DYNAMIC-EXTENT cleanup.
+(defstruct (cdynamic-extent (:include node)
+                            (:conc-name dynamic-extent-)
+                            (:predicate dynamic-extent-p)
+                            (:constructor make-dynamic-extent)
+                            (:copier nil))
+  ;; the values explicitly declared with this dynamic extent.
+  (values nil :type list)
+  ;; the cleanup for this extent. NULL indicates that this dynamic
+  ;; extent is over the environment and hence needs no cleanup code.
+  (cleanup nil :type (or cleanup null))
+  ;; some kind of info used by the back end
+  (info nil)
+  (preserve-info nil))
+
+(defprinter (cdynamic-extent :conc-name dynamic-extent-
+                             :identity t)
+  values
+  (info :test info))
+
 
 ;;;; miscellaneous IR1 structures
 
