@@ -65,8 +65,10 @@
   (:results (res :scs (unsigned-reg)))
   (:result-types positive-fixnum)
   (:generator 6
-    (inst lbz res x #+little-endian (- 2 other-pointer-lowtag)
-                    #+big-endian    (- 5 other-pointer-lowtag))
+    ;; convert ARRAY-RANK-POSITION to byte index and compensate for endianness
+    ;; ASSUMPTION: n-widetag-bits = 8 and rank is adjacent to widetag
+    (inst lbz res x #+little-endian (- 1 other-pointer-lowtag)
+                    #+big-endian    (- 6 other-pointer-lowtag))
     (inst addi res res 1)
     (inst andi. res res array-rank-mask)))
 
@@ -137,7 +139,7 @@
                (t
                 (inst addi ea index (- (ash vector-data-offset word-shift) other-pointer-lowtag))
                 (inst add ea object ea)))
-         (emit-gc-store-barrier object ea (list t1) (vop-nth-arg 2 vop) value)
+         (emit-gengc-barrier object ea (list t1) (vop-nth-arg 2 vop) value)
          (inst std value ea 0)))
     `(define-vop (,(symbolicate "DATA-VECTOR-SET/" (string type))
                   ,(symbolicate (string variant) "-SET"))
@@ -513,6 +515,15 @@
          (value :scs (unsigned-reg)))
   (:arg-types * positive-fixnum unsigned-num)
   (:variant vector-data-offset other-pointer-lowtag))
+
+;;; Weak vectors
+(define-vop (%weakvec-ref word-index-ref)
+  (:translate %weakvec-ref)
+  (:variant vector-data-offset other-pointer-lowtag))
+
+(define-vop (%weakvec-set word-index-set)
+  (:translate %weakvec-set)
+  (:variant vector-data-offset other-pointer-lowtag))
 
 ;;;
 
