@@ -11,22 +11,16 @@
  * files for more information.
  */
 
-/*
- * FIXME:
- *   Some of the code in here is deeply broken, depending on guessing
- *   already out-of-date values instead of getting them from sbcl.h.
- */
-
 #include <stdio.h>
 #include <string.h>
 
-#include "sbcl.h"
+#include "genesis/sbcl.h"
 #include "print.h"
 #include "runtime.h"
 #include "code.h"
 #include "gc.h"
 #include "genesis/gc-tables.h"
-#include "thread.h"              /* genesis/primitive-objects.h needs this */
+#include "thread.h"
 #include <errno.h>
 #include <stdlib.h>
 #include <inttypes.h>
@@ -833,14 +827,14 @@ struct vector * symbol_name(struct symbol* sym)
   lispobj name = sym->name;
   if (lowtag_of(name) != OTHER_POINTER_LOWTAG) return NULL;
   lispobj string = decode_symbol_name(name);
-  return VECTOR(follow_maybe_fp(string));
+  return VECTOR(follow_fp(string)); // can't have a nameless symbol
 }
 struct vector * classoid_name(lispobj * classoid)
 {
   if (forwarding_pointer_p(classoid))
       classoid = native_pointer(forwarding_pointer_value(classoid));
   // Classoids are named by symbols even though a CLASS name is arbitrary (theoretically)
-  lispobj sym = classoid[CLASSOID_NAME_WORDINDEX];
+  lispobj sym = ((struct classoid*)classoid)->name;
   return lowtag_of(sym) != OTHER_POINTER_LOWTAG ? NULL : symbol_name(SYMBOL(sym));
 }
 struct vector * layout_classoid_name(lispobj * layout)
