@@ -8,7 +8,7 @@
  * provided with absolutely no warranty. See the COPYING and CREDITS
  * files for more information.
  */
-#include "sbcl.h"
+#include "genesis/sbcl.h"
 
 #ifdef LISP_FEATURE_SB_SAFEPOINT /* entire file */
 #include <stdlib.h>
@@ -36,8 +36,8 @@
 #include "genesis/cons.h"
 #include "genesis/fdefn.h"
 #include "interr.h"
-#include "alloc.h"
-#include "gc-internal.h"
+#include "genesis/sap.h"
+#include "gc.h"
 #include "interrupt.h"
 #include "lispregs.h"
 #include "print.h"
@@ -99,7 +99,6 @@ const char* gc_phase_names[GC_NPHASES] = {
  * time will move to GC_INVOKED and then wait for the WITHOUT-GCING
  * thread to finish up, then proceed to GC_COLLECT. */
 
-#ifdef LISP_FEATURE_SB_THREAD
 #define CURRENT_THREAD_VAR(name) \
     struct thread *name = get_sb_vm_thread()
 #define THREAD_STOP_PENDING(th) \
@@ -107,14 +106,8 @@ const char* gc_phase_names[GC_NPHASES] = {
 #define SET_THREAD_STOP_PENDING(th,state) \
     write_TLS(STOP_FOR_GC_PENDING,state,th)
 #define WITH_ALL_THREADS_LOCK \
-    mutex_acquire(&all_threads_lock); \
-    RUN_BODY_ONCE(all_threads_lock, mutex_release(&all_threads_lock))
-#else
-#define CURRENT_THREAD_VAR(name)
-#define THREAD_STOP_PENDING(th) NIL
-#define SET_THREAD_STOP_PENDING(th,state)
-#define WITH_ALL_THREADS_LOCK
-#endif
+    ignore_value(mutex_acquire(&all_threads_lock)); \
+    RUN_BODY_ONCE(all_threads_lock, ignore_value(mutex_release(&all_threads_lock)))
 
 #if !defined(LISP_FEATURE_WIN32)
 /* win32-os.c covers these, but there is no unixlike-os.c, so the normal
@@ -178,7 +171,7 @@ void safepoint_init()
         InitializeConditionVariable(&gc_state.phase_cond[i]);
     InitializeCriticalSection(&gc_state.lock);
 #else
-    os_validate(NOT_MOVABLE, GC_SAFEPOINT_PAGE_ADDR, BACKEND_PAGE_BYTES, 0, 0);
+    os_alloc_gc_space(0, NOT_MOVABLE, GC_SAFEPOINT_PAGE_ADDR, BACKEND_PAGE_BYTES);
 #endif
     gc_state.phase = GC_NONE;
 }
@@ -187,10 +180,8 @@ void
 gc_state_lock()
 {
     odxprint(safepoints,"GC state to be locked");
-#ifdef LISP_FEATURE_SB_THREAD
-    int result = mutex_acquire(&gc_state.lock);
+    int __attribute__((unused)) result = mutex_acquire(&gc_state.lock);
     gc_assert(result);
-#endif
     if (gc_state.master) {
         fprintf(stderr,"GC state lock glitch [%p] in thread %p phase %d (%s)\n",
                 gc_state.master,get_sb_vm_thread(),gc_state.phase,
@@ -210,10 +201,8 @@ gc_state_unlock()
              gc_state.phase, gc_phase_names[gc_state.phase]);
     gc_assert(get_sb_vm_thread()==gc_state.master);
     gc_state.master = NULL;
-#ifdef LISP_FEATURE_SB_THREAD
-    int result = mutex_release(&gc_state.lock);
+    int __attribute__((unused)) result = mutex_release(&gc_state.lock);
     gc_assert(result);
-#endif
     odxprint(safepoints,"%s","GC state unlocked");
 }
 
@@ -228,13 +217,7 @@ gc_state_wait(gc_phase_t phase)
     gc_assert(gc_state.master == self);
     gc_state.master = NULL;
     while(gc_state.phase != phase && !(phase == GC_QUIET && (gc_state.phase > GC_QUIET))) {
-#ifdef LISP_FEATURE_WIN32
-        SleepConditionVariableCS(&gc_state.phase_cond[phase], &gc_state.lock, INFINITE);
-#elif defined LISP_FEATURE_SB_THREAD
-        pthread_cond_wait(&gc_state.phase_cond[phase],&gc_state.lock);
-#else
-        lose("gc_state_wait() blocks, but we're #-SB-THREAD");
-#endif
+        CONDITION_VAR_WAIT(&gc_state.phase_cond[phase], &gc_state.lock);
     }
     gc_assert(gc_state.master == NULL);
     gc_state.master = self;
@@ -283,10 +266,10 @@ static inline gc_phase_t gc_phase_next(gc_phase_t old) {
     return (old+1) % GC_NPHASES;
 }
 
-static inline boolean
+static inline bool
 thread_blocks_gc(struct thread *thread)
 {
-    return read_TLS(GC_INHIBIT,thread)==T;
+    return read_TLS(GC_INHIBIT,thread)==LISP_T;
 }
 /* set_thread_csp_access -- alter page permissions for not-in-Lisp
    flag (Lisp Stack Top) of the thread `th'. The flag may be modified
@@ -305,8 +288,8 @@ thread_blocks_gc(struct thread *thread)
    There is a full VM page reserved for this word; page permissions
    are switched to read-only for race-free examine + wait + use
    scenarios. */
-static inline boolean
-set_thread_csp_access(struct thread* th, boolean writable)
+static inline bool
+set_thread_csp_access(struct thread* th, bool writable)
 {
     os_protect((char*)th - (THREAD_HEADER_SLOTS*N_WORD_BYTES) - THREAD_CSP_PAGE_SIZE,
                THREAD_CSP_PAGE_SIZE,
@@ -336,20 +319,20 @@ static inline void gc_notify_early()
             if (p==gc_state.collector)
                 continue;
             odxprint(safepoints,"notifying thread %p csp %p",p,csp_around_foreign_call(p));
-            boolean was_in_lisp = !set_thread_csp_access(p,0);
+            bool was_in_lisp = !set_thread_csp_access(p,0);
             if (was_in_lisp) {
                 /* Threads "in-lisp" block leaving GC_MESSAGE, as we
                  * need them to hit their CSP or the GSP, and we unmap
                  * the GSP when transitioning to GC_INVOKED. */
                 gc_state.phase_wait[GC_MESSAGE]++;
-                SET_THREAD_STOP_PENDING(p,T);
+                SET_THREAD_STOP_PENDING(p, LISP_T);
             } else if (thread_blocks_gc(p)) {
                 /* Threads "in-alien" don't block leaving GC_MESSAGE,
                  * as the CSP trap is sufficient to catch them, but
                  * any thread that is WITHOUT-GCING prevents exit from
                  * GC_INVOKED. */
                 gc_state.phase_wait[GC_INVOKED]++;
-                SET_THREAD_STOP_PENDING(p,T);
+                SET_THREAD_STOP_PENDING(p, LISP_T);
             }
         }
     }
@@ -370,10 +353,10 @@ static inline void gc_notify_final()
             if (p == gc_state.collector)
                 continue;
             odxprint(safepoints,"notifying thread %p csp %p",p,csp_around_foreign_call(p));
-            boolean was_in_lisp = !set_thread_csp_access(p,0);
+            bool was_in_lisp = !set_thread_csp_access(p,0);
             if (was_in_lisp) {
                 gc_state.phase_wait[GC_SETTLED]++;
-                SET_THREAD_STOP_PENDING(p,T);
+                SET_THREAD_STOP_PENDING(p, LISP_T);
             }
         }
     }
@@ -383,12 +366,12 @@ static inline void gc_done()
 {
     CURRENT_THREAD_VAR(self);
     struct thread *p;
-    boolean inhibit = (read_TLS(GC_INHIBIT,self)==T);
+    bool inhibit = (read_TLS(GC_INHIBIT,self)==LISP_T);
 
     odxprint(safepoints,"%s","global denotification");
     WITH_ALL_THREADS_LOCK {
         for_each_thread(p) {
-            if (inhibit && (read_TLS(GC_PENDING,p)==T))
+            if (inhibit && (read_TLS(GC_PENDING,p)==LISP_T))
                 write_TLS(GC_PENDING,NIL,p);
             set_thread_csp_access(p,1);
         }
@@ -453,11 +436,7 @@ static inline void gc_advance(gc_phase_t cur, gc_phase_t old) {
         gc_state.phase = gc_phase_next(gc_state.phase);
         odxprint(safepoints,"no blockers, direct advance to %d (%s)",gc_state.phase,gc_phase_names[gc_state.phase]);
         gc_handle_phase();
-#ifdef LISP_FEATURE_WIN32
-        WakeAllConditionVariable(&gc_state.phase_cond[gc_state.phase]);
-#elif defined LISP_FEATURE_SB_THREAD
-        pthread_cond_broadcast(&gc_state.phase_cond[gc_state.phase]);
-#endif
+        CONDITION_VAR_WAKE_ALL(&gc_state.phase_cond[gc_state.phase]);
     }
     odxprint(safepoints,"going to wait for %d threads",gc_state.phase_wait[gc_state.phase]);
     gc_state_wait(cur);
@@ -470,7 +449,7 @@ thread_register_gc_trigger()
     struct thread *self = get_sb_vm_thread();
     WITH_GC_STATE_LOCK {
         if (gc_state.phase == GC_NONE &&
-            read_TLS(IN_SAFEPOINT,self)!=T &&
+            read_TLS(IN_SAFEPOINT,self)!=LISP_T &&
             !thread_blocks_gc(self)) {
             /* A thread (this thread), while doing allocation, has
              * determined that we need to run the garbage collector.
@@ -544,7 +523,7 @@ check_pending_thruptions(os_context_t *ctx)
      * "signal". */
     if (thread_extra_data(p)->pending_signal_set)
         if (__sync_fetch_and_and(&thread_extra_data(p)->pending_signal_set,0))
-            write_TLS(THRUPTION_PENDING, T, p);
+            write_TLS(THRUPTION_PENDING, LISP_T, p);
 #endif
 
     if (!thread_may_thrupt(ctx))
@@ -622,7 +601,7 @@ assert_on_stack(struct thread *th, void *esp)
 }
 
 /// Similar to the one in gc-common, but without the sigmask test.
-static boolean can_invoke_post_gc(struct thread* th)
+static bool can_invoke_post_gc(struct thread* th)
 {
     lispobj obj = th->lisp_thread;
     if (!obj) return 0;
@@ -639,25 +618,25 @@ int check_pending_gc(__attribute((unused)) os_context_t *ctx)
     int done = 0;
     sigset_t sigset;
 
-    if ((read_TLS(IN_SAFEPOINT,self) == T) &&
+    if ((read_TLS(IN_SAFEPOINT,self) == LISP_T) &&
         ((read_TLS(GC_INHIBIT,self) == NIL) &&
          (read_TLS(GC_PENDING,self) == NIL))) {
         write_TLS(IN_SAFEPOINT,NIL,self);
     }
     if (!thread_blocks_gc(self) && (read_TLS(IN_SAFEPOINT, self) == NIL)) {
-        if (read_TLS(GC_PENDING, self) == T) {
+        if (read_TLS(GC_PENDING, self) == LISP_T) {
             lispobj gc_happened = NIL;
 
-            bind_variable(IN_SAFEPOINT,T,self);
+            bind_variable(IN_SAFEPOINT,LISP_T,self);
             block_deferrable_signals(&sigset);
-            if(read_TLS(GC_PENDING,self)==T)
+            if(read_TLS(GC_PENDING,self)==LISP_T)
                 gc_happened = funcall1(StaticSymbolFunction(SUB_GC), 0);
             unbind(self);
             thread_sigmask(SIG_SETMASK,&sigset,NULL);
-            if (gc_happened == T) {
+            if (gc_happened == LISP_T) {
                 /* POST_GC wants to enable interrupts */
-                if ((read_TLS(INTERRUPTS_ENABLED,self) == T ||
-                     read_TLS(ALLOW_WITH_INTERRUPTS,self) == T)
+                if ((read_TLS(INTERRUPTS_ENABLED,self) == LISP_T ||
+                     read_TLS(ALLOW_WITH_INTERRUPTS,self) == LISP_T)
                     && can_invoke_post_gc(self))
                     funcall0(StaticSymbolFunction(POST_GC));
                 done = 1;
@@ -671,15 +650,15 @@ int check_pending_gc(__attribute((unused)) os_context_t *ctx)
 void thread_in_lisp_raised(os_context_t *ctxptr)
 {
     struct thread *self = get_sb_vm_thread();
-    boolean check_gc_and_thruptions = 0;
+    bool check_gc_and_thruptions = 0;
     odxprint(safepoints,"%s","thread_in_lisp_raised");
 
     /* Either we just hit the GSP trap, or we took a PIT stop and
      * there is a stop-for-GC or thruption pending. */
     WITH_GC_STATE_LOCK {
         if (gc_state.phase == GC_FLIGHT &&
-            read_TLS(GC_PENDING,self)==T &&
-            !thread_blocks_gc(self) && read_TLS(IN_SAFEPOINT,self)!=T) {
+            read_TLS(GC_PENDING,self)==LISP_T &&
+            !thread_blocks_gc(self) && read_TLS(IN_SAFEPOINT,self)!=LISP_T) {
             /* Some thread (possibly even this one) that does not have
              * GC_INHIBIT set has noticed that a GC is warranted and
              * advanced the phase to GC_FLIGHT, arming the GSP trap,
@@ -699,7 +678,7 @@ void thread_in_lisp_raised(os_context_t *ctxptr)
                 gc_advance(GC_NONE,GC_QUIET);
             } else {
                 /* ??? Isn't this already T? */
-                write_TLS(GC_PENDING,T,self);
+                write_TLS(GC_PENDING,LISP_T,self);
             }
             csp_around_foreign_call(self) = 0;
             check_gc_and_thruptions = 1;
@@ -732,7 +711,7 @@ void thread_in_lisp_raised(os_context_t *ctxptr)
                  * we'll take a PIT stop and wind up in the case
                  * above...  Or we'll call gc_stop_the_world(). */
                 gc_advance(GC_INVOKED,gc_state.phase);
-                SET_THREAD_STOP_PENDING(self,T);
+                SET_THREAD_STOP_PENDING(self,LISP_T);
                 /* Why do we not want to run thruptions here? */
             }
         }
@@ -750,7 +729,7 @@ void thread_in_lisp_raised(os_context_t *ctxptr)
 void thread_in_safety_transition(os_context_t *ctxptr)
 {
     struct thread *self = get_sb_vm_thread();
-    boolean was_in_alien;
+    bool was_in_alien;
 
     odxprint(safepoints,"%s","GC safety transition");
     WITH_GC_STATE_LOCK {
@@ -795,7 +774,7 @@ void thread_in_safety_transition(os_context_t *ctxptr)
                  * logic is identical to the similar case in
                  * thread_in_lisp_raised(). */
                 gc_advance(GC_INVOKED,gc_state.phase);
-                SET_THREAD_STOP_PENDING(self,T);
+                SET_THREAD_STOP_PENDING(self,LISP_T);
             }
         }
     }
@@ -810,7 +789,7 @@ void thread_in_safety_transition(os_context_t *ctxptr)
 void thread_interrupted(os_context_t *ctxptr)
 {
     struct thread *self = get_sb_vm_thread();
-    boolean gc_active, was_in_alien;
+    bool gc_active, was_in_alien;
 
     odxprint(safepoints,"%s","pending interrupt trap");
     WITH_GC_STATE_LOCK {
@@ -852,6 +831,7 @@ gc_stop_the_world()
         switch(gc_state.phase) {
         case GC_NONE:
             gc_advance(GC_QUIET,gc_state.phase);
+            /* FALLTHRU */
         case GC_FLIGHT:
         case GC_MESSAGE:
         case GC_INVOKED:
@@ -866,6 +846,7 @@ gc_stop_the_world()
             } else {
                 gc_state_wait(GC_QUIET);
             }
+            /* FALLTHRU */
         case GC_QUIET:
             /* Some number of threads were trying to get to GC_QUIET.
              * But this thread is sufficient to be able to leave
@@ -915,13 +896,13 @@ void wake_thread_impl(struct thread_instance *lispthread)
     struct thread* thread = (void*)lispthread->uw_primitive_thread;
     wake_thread_io(thread);
 
-    if (read_TLS(THRUPTION_PENDING,thread)==T)
+    if (read_TLS(THRUPTION_PENDING,thread)==LISP_T)
         return;
 
-    write_TLS(THRUPTION_PENDING,T,thread);
+    write_TLS(THRUPTION_PENDING,LISP_T,thread);
 
-    if ((read_TLS(GC_PENDING,thread)==T)
-        ||(THREAD_STOP_PENDING(thread)==T)
+    if ((read_TLS(GC_PENDING,thread)==LISP_T)
+        ||(THREAD_STOP_PENDING(thread)==LISP_T)
         )
         return;
 
@@ -947,7 +928,7 @@ void wake_thread_impl(struct thread_instance *lispthread)
     /* Must not and need not attempt to signal ourselves while we're the
      * STW initiator. */
     if (thread == self) {
-        write_TLS(THRUPTION_PENDING,T,self);
+        write_TLS(THRUPTION_PENDING,LISP_T,self);
         while (check_pending_thruptions(0 /* ignore the sigmask */))
             ;
         return;
@@ -972,9 +953,9 @@ void wake_thread_impl(struct thread_instance *lispthread)
                 WITH_ALL_THREADS_LOCK {
                     do {
                             odxprint(safepoints, "wake_thread_posix: found");
-                            write_TLS(THRUPTION_PENDING,T,thread);
-                            if (read_TLS(GC_PENDING,thread) == T
-                                || THREAD_STOP_PENDING(thread) == T)
+                            write_TLS(THRUPTION_PENDING,LISP_T,thread);
+                            if (read_TLS(GC_PENDING,thread) == LISP_T
+                                || THREAD_STOP_PENDING(thread) == LISP_T)
                                 break;
 
                             if (os_get_csp(thread)) {
@@ -992,7 +973,7 @@ void wake_thread_impl(struct thread_instance *lispthread)
             gc_advance(GC_NONE,GC_INVOKED);
         } else {
             odxprint(safepoints, "wake_thread_posix: passive");
-            write_TLS(THRUPTION_PENDING, T, thread);
+            write_TLS(THRUPTION_PENDING, LISP_T, thread);
         }
     }
     thread_sigmask(SIG_SETMASK, &oldset, 0);
