@@ -16,7 +16,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "sbcl.h"
+#include "genesis/sbcl.h"
 #include "runtime.h"
 #include "os.h"
 #include "globals.h"
@@ -32,6 +32,9 @@ uword_t DYNAMIC_SPACE_START;
 #ifndef LISP_FEATURE_DARWIN_JIT
 uword_t READ_ONLY_SPACE_START, READ_ONLY_SPACE_END;
 #endif
+#ifdef LISP_FEATURE_RELOCATABLE_STATIC_SPACE
+uword_t STATIC_SPACE_START, STATIC_SPACE_END;
+#endif
 
 uword_t asm_routines_start, asm_routines_end;
 
@@ -81,7 +84,7 @@ ensure_undefined_alien(void) {
     }
 }
 
-boolean allocate_hardwired_spaces(boolean hard_failp)
+bool allocate_hardwired_spaces(bool hard_failp)
 {
     struct {
         uword_t start;
@@ -117,7 +120,7 @@ boolean allocate_hardwired_spaces(boolean hard_failp)
 }
 
 void
-allocate_lisp_dynamic_space(boolean did_preinit)
+allocate_lisp_dynamic_space(bool did_preinit)
 {
     // Small spaces can be allocated after large spaces are.
     // The above code is only utilized when heap relocation is disabled.
